#!/usr/bin/python

import sys, os, re, traceback, string, socket
from optparse import OptionParser, OptionGroup

# add path to modules. It must be first, because we're using more recent standard modules (tempfile) then those in default 2.2.3 installation.
if sys.platform != 'win32':
	sys.path = [os.path.join(os.path.realpath(sys.path[0]), 'modules')] + sys.path
else:
	sys.path = sys.path + [os.path.join(os.path.realpath(sys.path[0]), 'modules')]

import uConfig, uSysDB, uUtil
import ConfigParser

def findDBConfig():
	default_rv = {
		'dbhost' : socket.gethostname(),
		'user' : 'plesk',
		'password' : 'plesk'
	}

	if sys.platform == 'win32':
		return 	default_rv
	else:
		etc_dir = "/usr/local/pem/etc"
		kernel_conf_path = os.path.join(etc_dir, "Kernel.conf")
		odbc_ini_path = os.path.join(etc_dir, "odbc.ini")

		if not os.path.isfile(kernel_conf_path) or not os.path.isfile(odbc_ini_path):
			return default_rv

		kernel_conf = file(kernel_conf_path)
		pp = uUtil.readPropertiesFile(kernel_conf)
		kernel_conf.close()

		odbc = ConfigParser.ConfigParser()
		odbc.read(odbc_ini_path)

		dsnname = pp["dsn"]
		rv = {}
		rv['dbhost'] = odbc.get(dsnname, "servername")
		rv['user'] = pp["dsn_login"]
		rv['password'] = pp['dsn_passwd']
		return rv

parser = OptionParser()

# declare options
common_opts = OptionGroup(parser, "Commands and common options")

common_opts.add_option("--subs-list", dest="sub_list",
	help="The comma-separated list of subscriptions ids. ALL keyword to upgrade all Exchange subscriptions.\nSamples:\n\t./exch2k10sp2_upgrade.py --subs-list 100, 800\n\t./exch2k10sp2_upgrade.py --subs-list ALL")

common_opts.add_option("--subs-file", dest="subs_infile",
	help="The file contains lines with one subscription id per line.\nSamples:\n\t./exch2k10sp2_upgrade.py --subs-file subs.txt")

common_opts.add_option("--upgrade-list", dest="upgrade_list", action="store_true",
	help="Lists all subscriptions that are ready for upgrade.")

common_opts.add_option("--set-upgrade-limit", dest="set_upgrade_limit",
	help="Set the maximum number of Address List upgrade tasks, which can be executed simultaneously.\nSamples:\n\t./exch2k10sp2_upgrade.py --set-upgrade-limit 100",
	metavar="LIMIT")

common_opts.add_option("--get-upgrade-limit", dest="get_upgrade_limit", action="store_true",
	help="Return the maximum number of Address List upgrade tasks, which can be executed simultaneously.")

dbconfig = findDBConfig()
	
common_opts.add_option("--db-user", "-u", dest="db_user",
	help="(E,I) database user to access POA database", metavar="USER", default=dbconfig['user'])
common_opts.add_option("--db-password", "-p", dest="db_password",
	help="(E,I) database password to access POA database", metavar="PASSWORD", default=dbconfig['password'])

common_opts.add_option("--db-host", "-s", dest="db_host",
	help="(E,I) database host to access POA database", metavar="DBHOST", default=dbconfig['dbhost'])

parser.add_option_group(common_opts)

options, arg_list = parser.parse_args()

# check options
if not options.sub_list and not options.subs_infile and not options.upgrade_list and not options.get_upgrade_limit and not options.set_upgrade_limit:
	parser.print_help()
	sys.exit(1)
	
if options.upgrade_list	and (options.sub_list or options.subs_infile or options.get_upgrade_limit or options.set_upgrade_limit):
	print "--upgrade-list must be used separetly from other options. Please remove superfluous option."
	sys.exit(4)

if options.set_upgrade_limit and (options.sub_list or options.subs_infile or options.upgrade_list or options.get_upgrade_limit):
	print "--set-upgrade-limit must be used separetly from other options. Please remove superfluous option."
	sys.exit(4)

if options.get_upgrade_limit and (options.sub_list or options.subs_infile or options.upgrade_list or options.set_upgrade_limit):
	print "--get-upgrade-limit must be used separetly from other options. Please remove superfluous option."
	sys.exit(4)
	
if options.sub_list and options.subs_infile:
	print "--subs-list and --subs-file options are mutually exclusive. Please remove superfluous option."
	sys.exit(4)
	
# Actually we don't want use this POA Config class but we need fill this
# via other updater modules requirements
config = uConfig.Config()

config.assign(["database", "host"], options.db_host)
config.assign(["database", "user"], options.db_user)
config.assign(["database", "password"], options.db_password)

try:
	uSysDB.init(config)
except Exception, e:
	print "Cannot access to POA DB: "
	print e
	if not options.db_user or not options.db_password:
		print "Specify correct login (--db_user) and password (--db-password) for POA DB, these parameters are mandatory."
	else:
		print "Check POA DB login and password"
	sys.exit(2)


con = uSysDB.connect()
cur = con.cursor()

if options.upgrade_list:
	sql = """
	SELECT ds.subscription_id, ds.windows_domain, ds.exchange_version FROM exch_ds_for_upgrade_al dsu
		INNER JOIN exch_domain_services ds ON (ds.ds_id=dsu.ds_id)
		WHERE dsu.upgrade_al_task_id IS NULL AND dsu.force_upgrade = 'n'"""
	cur.execute(sql)
	
	print "\nList of ready for upgrade subscriptions:\n"
	print "-"*80
	print "subscription id\t\t|windows domain\t\t|exchange version"
	print "-"*80
	for (sub_id, win_domain, exch_ver) in cur.fetchall():
		print "%d\t\t|%s\t\t|%d" % (sub_id, win_domain, exch_ver)
	sys.exit(0)

if options.set_upgrade_limit:
	if options.set_upgrade_limit.isdigit() != 1:
		print "Value of new limit of simultaneously Address List upgrade tasks '%s' is invalid." % options.set_upgrade_limit
		sys.exit(3)

	cur.execute("SELECT COUNT(1) FROM exch_exchange_sys_cfg WHERE name = 'sys.upgrade.als.concurrent.orgs.limit'")
	if cur.fetchone()[0] == 0:
		cur.execute("INSERT INTO exch_exchange_sys_cfg (name, value) VALUES ('sys.upgrade.als.concurrent.orgs.limit', %s)", str(options.set_upgrade_limit))
	else:
		cur.execute("UPDATE exch_exchange_sys_cfg SET value = %s WHERE name = 'sys.upgrade.als.concurrent.orgs.limit'", str(options.set_upgrade_limit))
	con.commit()

	print "\nThe maximum number of Address List upgrade tasks, which can be executed simultaneously is updated.\n"
	sys.exit(0)

if options.get_upgrade_limit:
	sql = "SELECT value FROM exch_exchange_sys_cfg WHERE name = 'sys.upgrade.als.concurrent.orgs.limit'"
	cur.execute(sql)
	rs = cur.fetchone()
	upgrade_limit = 10
	if rs:
		upgrade_limit = rs[0]

	print "\nThe maximum number of Address List upgrade tasks, which can be executed simultaneously: %s" % str(upgrade_limit)
	sys.exit(0)
	
	
sql = "UPDATE exch_ds_for_upgrade_al SET force_upgrade = 'y'"

sub_list = options.sub_list
	
if sub_list and sub_list != "ALL":
	sub_list = sub_list.split(',')
elif options.subs_infile:
	print "\nReading file '%s' for subscription list..." % options.subs_infile
	text_file = open(options.subs_infile, "r")
	sub_list = text_file.readlines()
	text_file.close()

# check ids
whereSql = ""
if sub_list != "ALL":
	for org_id in sub_list:
		org_id = org_id.strip()
		if org_id.isdigit() != 1:
			print "Subscription id '%s' is invalid." % org_id
			sys.exit(3)
	whereSql = " WHERE ds_id IN (select ds_id FROM exch_domain_services WHERE subscription_id IN (%s))" % ", ".join([sub_id for sub_id in sub_list])

sql += whereSql
cur.execute(sql)

if whereSql == "":
	whereSql = " WHERE "
else:
	whereSql += " AND "
cur.execute("SELECT COUNT(1) FROM exch_ds_for_upgrade_al " + whereSql + " force_upgrade = 'y'")
scheduledUpgradesCount = cur.fetchone()[0]
if not scheduledUpgradesCount:
	scheduledUpgradesCount = 0

con.commit()

if sub_list != "ALL" and scheduledUpgradesCount < len(sub_list):
	print "Some of subscriptions are absent or wrong id was passed via --sub-list parameter."
	
if scheduledUpgradesCount is None:
	scheduledUpgradesCount = 0
	
upgradingCount = 0
waitingForUpgradeCount = 0

sql = """
	SELECT COUNT(sub_q.ds_id) as cnt,
		sub_q.is_null as is_null
	FROM (
		SELECT ds_id, 
			CASE WHEN upgrade_al_task_id IS NULL THEN 1 ELSE 0 END AS is_null
		FROM exch_ds_for_upgrade_al ) sub_q
	GROUP BY sub_q.is_null"""
	
cur.execute(sql)
for (cnt, is_null) in cur.fetchall():
	if is_null == 0:
		upgradingCount = cnt
	else:
		waitingForUpgradeCount = cnt
		
print "---------------------------------------------------------------------------------------------------"
if waitingForUpgradeCount == 0:
	if upgradingCount == 0:
		print "All Exchange organization(s) has(ve) been upgraded.\n"
	else:
		print "%d Exchange organization(s) is(are) upgrading. There are no organizations to upgrade.\n" % upgradingCount
else:
	print "%d Exchange organization(s) is(are) upgrading. There are no organizations to upgrade.\n" % upgradingCount
	print "%d Exchange organization(s) is(are) waiting for the upgrade.\n" % (waitingForUpgradeCount - scheduledUpgradesCount)

print "Done."
