#!/usr/bin/python

from uDLCommon import *
from xml.dom import minidom as dom


class Module:
	def __init__(self, name, version, description, source):
		self.name = name
		self.version = version
		self.description = description
		self.source = source
		self.packages = []
		self.native_packages = []

	def add_package(self, node):
		self.packages.append(PkgInstallation(node = node))
	
	def add_native_package(self, node):
		self.native_packages.append(parseNativePackageNode(node))

	def unknown_element(self, node):
		raise Exception("%s: unknown element", node.name)
	
	def __repr__(self):
		return "<Module %s-%s from '%s'>" % (self.name, self.version, self.source)

xml_mapping = {
    "PACKAGE" : Module.add_package,
    "NATIVE_PACKAGE" : Module.add_native_package
};

def readModuleFromDOM(doc, name):
	root = doc.documentElement
	rv = Module(root.getAttribute("id"), root.getAttribute("version"), root.getAttribute("name"), name)
	content = doc.getElementsByTagName("CONTENT")
	if not content:
		raise Exception("No CONTENT module in %s", name)
	
	for child in content[0].childNodes:
		if child.nodeType != dom.Node.ELEMENT_NODE:
			continue

		xml_mapping.get(child.tagName, Module.unknown_element)(rv, child)

	return rv

def readModule(fn):
	doc = dom.parse(fn)
	return readModuleFromDOM(doc, fn)

def getInstalledModules(con):
	cur = con.cursor()
	
	# Platform goes first, and always consdired to be installed
	q = "SELECT mb.data, m.name FROM modules_body mb JOIN modules m ON (mb.module_id = m.module_id) WHERE "
	cur.execute(q + "m.name = 'Platform'")
	row = cur.fetchone()
	if not row:
		raise Exception("There is no Platform module")
	rv = [ readModuleFromDOM(dom.parseString(str(row[0])), 'Platform') ]

	cur.execute(q + "m.name != 'Platform' AND m.status != 'n'")

	return rv + [ readModuleFromDOM(dom.parseString(str(row[0])), row[1]) for row in cur.fetchall() ]

if __name__ == '__main__':
	import sys
	for a in sys.argv[1:]:
		print readModule(a)
