#!/usr/bin/python
import uDialog, re, uLogging, os, socket, sys, uLinux

def showLicense(title, license):
	print title
	print license
	return uDialog.askYesNo("Do you agree with terms of license?")


def askNetInterface(ifaces, question):
	choices = {}
	print question
	for i in xrange(0, len(ifaces)):
		choices[i] = ifaces[i]
	idx = uDialog.select(("No", "Interface", "IP Address"), choices)

	return ifaces[idx]

def initialize():
	pass

def set_net_iface(props, prefix, iface):
	name, ip, mask = iface
	props['%s_netiface_name' % prefix ] = name
	props['%s_ip' % prefix ] = ip
	props['%s_netmask' % prefix ] = mask
	

only_letters_pattern = re.compile(r"[a-z]+")
email_pattern = re.compile(r"(\w|-)+@(\w|-)+\.(\w|-)+")
hostname_pattern = re.compile(r"(\w|-)+\.(\w|-)+")

def configure(default_parameters):
	rv = default_parameters.copy()
	all_net_ifaces = uLinux.listNetifaces()
	hostname = socket.gethostname()

	good_net_ifaces = [ x for x in all_net_ifaces if x[1] != '127.0.0.1']
	if not good_net_ifaces:
		raise Exception("No net interfaces found")
	elif len(good_net_ifaces) > 1:
		internal = askNetInterface(good_net_ifaces, "Please select communication interface")
		external = askNetInterface(good_net_ifaces, "Please select external interface")
	else:
		uLogging.notice("Only one IP address configured, it will be used as both external and internal one")
		external = internal = good_net_ifaces[0]
	set_net_iface(rv, "communication", internal)
	set_net_iface(rv, "external", external)

	rv['dsn_login'] = uDialog.ask("Database username", rv['dsn_login'], lambda x: only_letters_pattern.match(x))
	rv['db'] = uDialog.ask("Database name", rv['db'], lambda x: only_letters_pattern.match(x))
	rv['dsn'] = 'plesk'
	rv['sc-MailMessenger:messenger_mail.from'] = uDialog.ask("From email address for notification messages", os.environ.get("USER", "root") + '@' + socket.gethostname(), lambda x: email_pattern.match(x))
	rv['sc-MailMessenger:messenger_mail.smtphost'] = uDialog.ask("SMTP server to send notification messages", hostname, lambda x : hostname_pattern.match(x))

	rv['hostname'] = uDialog.ask("FQDN of this host", hostname, lambda x: hostname_pattern.match(x))
	print "If you want to use different host as DB server, PostgreSQL should be already installed, configured and started there"
	rv['db_hostname'] = uDialog.ask("Database server name", hostname, lambda x: hostname_pattern.match(x))

	return rv

class InstallationProgress: # dummy klass, show kein progress
	def __init__(self):
		pass
	
	def set_progress(self, dummy1, dummy2 = None):
		pass

	def new_phase(self, dummy):
		pass

	def end_phase(self):
		pass


def choose_modules(build, props):
	rv = []
	mprop = props.get("modules", "")
	if mprop.lower() == "all":
		default_modules = [ m.name for m in build.modules ]
	else:
		default_modules = [ m.strip() for m in props.get("modules", "").split(",")]
	for module in build.modules:
		if module.name != 'Platform' and uDialog.askYesNo("Do you want to install %s?" % module.description, module.name in default_modules):
			rv.append(module.name)

	return rv

def askYesNo(question, default = None):
	return uDialog.askYesNo(question, default)


def message(title, text):
	print 
	print "=" * 70
	print title
	print text
	print "=" * 70
	print

def congrat(text):
	uLogging.notice(text)
	message("CONGRATULATIONS!", text)

def error(text):
	uLogging.err(text)
	message("ERROR", text)
