from uLogging import *
import time
import code
import sys, cStringIO, traceback
import uDialog

def pretty_timing(totals):
	mins, secs = divmod(abs(totals), 60)
	mins = int(mins)
	hours, mins = divmod(mins, 60)
	time_str = '%s sec' % secs
	if mins or hours:
		time_str = '%s min, ' % mins + time_str
		if hours:
			time_str = '%s hour, ' % hours + time_str
	if totals < 0:
		time_str = "-" + time_str
	return time_str

class Progress:
	def __init__(self):
		self.whats= []
	
	def do(self, what, *args):
		now = time.time()
		self.whats.append((now, what, args))
		if what:
			msg = what[0].capitalize() + what[1:]
		else:
			msg = ""
		notice("[%s] %s=> %s" , time.asctime(), '='*len(self.whats), msg % args)

	def done(self, success = True):
		now = time.time()
		if not self.whats:
			err("Internal error / unmatched do/done")
			return
		st, what, args = self.whats[-1]
		if not success and what:
			what = what[0].capitalize() + what[1:]
		del self.whats[-1]
		wargs = what % args
		totals = now - st
		time_str = pretty_timing(totals)


		if success:
			notice("[%s] <=%s Finished %s (%s)", time.asctime(), '='*(len(self.whats)+1), wargs, time_str)
		else:
			notice("[%s] <=%s %s failed (%s)", time.asctime(), '='*(len(self.whats)+1), wargs, time_str)


	def what(self):
		if not self.whats:
			return None
		else:
			st, what, args = self.whats[-1]
			try :
				return what % args
			except TypeError:
				return "%s %s" % (what, args)

import re
retry_pattern=re.compile(r"^\s*r(e|et|etr|etry)?\s*$", re.IGNORECASE)
ignore_pattern=re.compile(r"\s*i(g|gn|gno|gnor|gnore)?\s*", re.IGNORECASE)
abort_pattern=re.compile(r"\s*a(b|bo|bor|bort)?\s*", re.IGNORECASE)
console_pattern=re.compile(r"\s*c(o|on|ons|onso|onsol|onsole)?\s*", re.IGNORECASE)

class ActionIgnored:
# not derived from Exception intentionally
	def __init__(self):
		pass

def format_exception():
	out = cStringIO.StringIO()
	traceback.print_exc(file=out)
	return out.getvalue()

def timed(fun, progress, what, *wargs):
	def fn(*args):
		progress.do(what, *wargs)
		try:
			rv =  fun(*args)
			progress.done()
			return rv
		except:
			progress.done(False)
			raise
	return fn

default_error_action = None

def retriable(fun, raise_on_ignore = False, allow_console = None):
	""" Automatically retries fun on Error """
	def fn(*args, **kwds):
		default_tried = False
		while True:
			try:
				return fun(*args, **kwds)
			except (KeyboardInterrupt, Exception), e:
				print_trace = True
				if isinstance(e, KeyboardInterrupt):
					err_msg = "Interrupted"
					print_trace = False
				elif not len(e.args):
					err_msg = e.__class__.__name__
				else:
					err_msg = ' '.join([str(x) for x in e.args])
				if print_trace:
					err("Error trace: %s", format_exception())
				err("%s while %s", err_msg, progress.what())
				answer = None
				question = "(A)bort/(R)etry/(I)gnore"
				if allow_console:
					question += "/(C)onsole"
				question += ": "
				while not answer:
					if default_error_action and not default_tried:
						answer = default_error_action
						default_tried = True
					else:
						answer= uDialog.input(question)
					if abort_pattern.match(answer):
						raise
					elif ignore_pattern.match(answer):
						if raise_on_ignore:
							raise ActionIgnored()
						else:
							return None
					elif retry_pattern.match(answer):
						pass
					elif allow_console and console_pattern.match(answer):
						console = code.InteractiveConsole(allow_console)
						answer=None
						try :
							console.interact("Type sys.exit() when you're done")
						except:
							pass
			
					else:
						answer= None
			except:
				err("not exception raised")
				global last_exc
				last_exc = sys.exc_info()
				print >> sys.stderr, sys.exc_info()
				raise
	return fn

def set_default_error_action(act):
	global default_error_action
	default_error_action = act

def get_default_error_action():
	global default_error_action
	return default_error_action

def nothrowable(fun):
	def nf(*args):
		try:
			return fun(*args)
		except:
			return None
	return nf

last_exc = None

progress=Progress()

__all__ = [ "progress", "retriable", "ActionIgnored", "nothrowable", "pretty_timing", "timed", "set_default_error_action", "get_default_error_action" ]
