from sets import *
from uLogging import *
import md5
import os.path
import re

def compare_versions(version_one, version_two):
	ds1 = version_one.split('.')
	ds2 = version_two.split('.')
	
	index = 0
	max_ind = min(len(ds1), len(ds2))

	while index < max_ind:
		if ds1[index] == ds2[index]:
			index += 1
			continue
		try:
			i1 = int(ds1[index])
			i2 = int(ds2[index])
			if i1 < i2:
				return -1
			elif i1 > i2:
				return 1
		except:
			pass

		s1 = ds1[index].split('-')
		s2 = ds2[index].split('-')
		index = 0
		max_ind = min(len(s1), len(s2))
		while index < max_ind:
			if s1[index] == s2[index]:
				index += 1
				continue
			try:
				i1 = int(s1[index])
				i2 = int(s2[index])
				if i1 < i2:
					return -1
				elif i1 > i2:
					return 1
			except:
				pass

			if s1 < s2:
				return -1
			else:
				return 1
		if len(s1) > len(s2):
			return 1
		elif len(s2) > len(s1):
			return -1
	
	if len(ds1) == len(ds2):
		return 0
	elif len(ds1) > len(ds2):
		return 1
	else:
		return -1

class Platform:
	def __init__(self, os, osver, arch='i386'):
		self.os = os
		self.osver = osver
		self.arch = arch

	def __str__(self):
		return "%s-%s-%s" % (self.os, self.osver, self.arch)
	
	def __repr__(self):
		return str(self)

	def __eq__(self, other):
		return self.os == other.os and self.osver == other.osver and self.arch == other.arch

	def __ne__(self, other):
		return not self.__eq__(other)

	def __hash__(self):
		return hash(str(self))

def is_platform_compat(p1, p2):
	if p1 is None or p2 is None or p1 == p2:
		return True

	if p1.osver == p2.osver:
		return p1.os == 'CentOS' and p2.os.startswith('RHE') or \
		       p1.os.startswith('RHE') and p2.os == 'CentOS' or \
		       p1.os == p2.os and p1.arch == 'i386'

class Package:
	def __init__(self, name, ctype, platform, is_single = True, is_custom = False):
		self.name = name
		self.ctype = ctype
		self.platform = platform
		self.is_single = is_single
		self.is_custom = is_custom
	
	def __str__(self):
		return "%s-%s-%s" % (self.ctype, self.platform, self.name)
	
	def __repr__(self):
		return str(self)

	def __eq__(self, other):
		return self.name == other.name and self.ctype == other.ctype and self.platform == other.platform

	def __hash__(self):
		return hash(str(self))

class RPM:
	def __init__(self, name, platform, filename = None):
		self.name = name
		self.platform = platform
		self.filename = filename
	
	def __str__(self):
		return "%s-%s" % (self.platform, self.name)
	
	def __repr__(self):
		return str(self)

	def __eq__(self, other):
		return self.name == other.name and  self.platform == other.platform

	def __hash__(self):
		return hash(str(self))

# pa50-pleskd-redhat-el5-x86-ez-20100216-1.noarch.rpm: <name>-<os>-<osver>-<arch>-ez-*
ez_template_pattern = re.compile(r'pa([0-9]+)-(.+)-(\w+)-(\w+)-(\w+)-ez-.+')

# vzpem-mysql5-client-as4-template-20090616-1.0-1.i386.rpm: <name>-<osver>[-<arch>]-template-*
std_template_pattern = re.compile(r'(.+)-(\w+)-()template-([0-9]+)-.+')
std_x64_template_pattern = re.compile(r'(.+)-(\w+)-(x86_64)-template-([0-9]+)-.+')

vz_template_os_pattern = re.compile(r'([a-z]*)([0-9]+)')

class VZTemplate:
	def __init__(self, fn):
		self.full_filename = fn
		self.filename = os.path.basename(self.full_filename)
		self.template_id = None
		ez = ez_template_pattern.match(self.filename)
		if ez:
			self.is_ez = True
			self.name = ez.group(2)
			self.version = ez.group(1)
			self.template_name = ".pa%s-%s" % (self.version, self.name)
			if len(self.version) != 2:
				# scheduleTemplatesInstallation in "update" script
				# does not handle non 2-digit versions well
				raise Exception("%s: Product version (%s) should be 2-digit number" %
					(self.full_filename, self.version))
			vz_os = ez.group(3)
			vz_osver = ez.group(4)
			vz_arch = ez.group(5)

			# compute POA platform
			if vz_os == 'redhat':
				_os = 'RHEL'
			elif vz_os == 'centos':
				_os = 'CentOS'
			else:
				raise Exception("%s: Unsupported template OS %s" %
					(self.full_filename, osver))
			v = vz_template_os_pattern.match(vz_osver)
			if not v:
				raise Exception("%s: Can not parse template OS version %s" %
					(self.full_filename, vz_osver))
			osver = v.group(2)
			if _os == 'RHEL' and int(osver) < 5:
				_os = 'RHES'
			arch = vz_arch
			if arch == 'x86':
				arch = 'i386'
			self.platform = Platform(_os, osver, arch)

			# compute POA VZ template set name
			self.set_name = "%s.%s-%s-%s" % (self.template_name, vz_os, vz_osver, vz_arch)
			return

		self.is_ez = False
		std = std_x64_template_pattern.match(self.filename)
		if not std:
			std = std_template_pattern.match(self.filename)
			if not std:
				raise Exception("%s: Template filename does not match neither std nor EZ template name pattern" % self.full_filename)

		vz_os = std.group(2)
		self.name = "%s-%s" % (std.group(1), vz_os)
		self.version = std.group(4)

		# compute POA platform
		v = vz_template_os_pattern.match(vz_os)
		if not v:
			raise Exception("%s: Can not parse template OS %s" % (self.full_filename, vz_os))
		_os = v.group(1)
		osver = v.group(2)
		if _os == 'as':
			_os = 'RHEL'
		elif _os == 'fc':
			_os = 'Fedora'
		elif _os == 'ce':
			_os = 'CentOS'
		else:
			raise Exception("%s: Unsupported template OS %s" % (self.full_filename, _os))
		if _os == 'RHEL' and int(osver) < 5:
			_os = 'RHES'
		if std.group(3):
			arch = 'x86_64'
			self.name += "-%s" % arch
		else:
			arch = 'i386'
		self.platform = Platform(_os, osver, arch)
		self.template_name = self.name
		self.set_name = self.name

	def __str__(self):
		s = "%s (%s/%s:%s): %s, %s (%s)" % (self.name,
			self.template_name, self.version, self.set_name,
			self.platform, self.filename, self.full_filename)
		return s

class BuiltRPM:
	def __init__(self, rpm, version, release, content):
		self.rpm = rpm
		self.version = version
		self.release = release
		self.content = content

	def __str__(self):
		if self.content is not None:
			return self.content.filename
		else:
			return str(self.rpm)
	
	def __repr__(self):
		return str(self)

class BuiltPackage:
	def __init__(self, package, version, content = None):
		self.package = package
		self.version = version
		self.content = content
		self.topdir = None
		self.old = None
		self.pkg_id = None
		self.build_tarball = None, None

	def __str__(self):
		return "%s-%s" % (self.package, self.version)
	
	def __repr__(self):
		return str(self)

	def __eq__(self, other):
		return self.package == other.package and self.version == other.version and (self.content is None and other.content is None or self.content.files == other.content.files)

	def __hash__(self):
		return hash(str(self))

	def findTarball(self, build):
		if self.build_tarball[0] != build:
			self.build_tarball = build, build.find_valid_tarball(self)
		return self.build_tarball[1]

class BuiltPackageContent:
	def __init__(self, filename, checksum, files):
		self.filename = filename
		self.checksum = checksum
		self.files = files

class WindowsUpdate:
	def __init__(self):
		self.installer_path = None
		self.async_exec_path = None
		self.perform = True

def describe_difference(old, new):
	newfiles = Set(new.iterkeys()); oldfiles = Set(old.iterkeys())
	added = list(newfiles - oldfiles)
	removed = list(oldfiles - newfiles)
	changed = [ f for f in newfiles & oldfiles if old[f] != new[f] ]

	return '\n'.join(["%s file(s): %s" % (kind, ", ".join(lst)) for kind, lst in [ ("Added", added), ("Removed", removed), ("Changed", changed)] if lst])

class Build:
	""" Represents build characteristics """
	def __init__(self, name, conf = None):
		""" Creates new empty Build with name """
		self.name = name
		self.platforms = Set()
		self.contents = {}
		self.config = conf
		self.rpms = {}
		self.topdir = None
		self.tarballs_locations = {}
		self.platform_binaries = {}
		self.windows_update = WindowsUpdate()

		self.modules = []
		self.vz_templates = []

		self.files = {}

	def add_module(self, module):
		self.modules.append(module)

	def __add_package(self, built_package):
		self.contents[built_package.package] = built_package

	def add_package(self, built_package, fail_on_different_versions = False):
		self.platforms.add(built_package.package.platform)
		if self.contents.has_key(built_package.package):
			if self.config and self.config.warning.duplicates:
				warn("%s: duplicate package in build %s", built_package, self.name)
			vc = compare_versions(built_package.version, self.contents[built_package.package].version)
			if vc != 0 and fail_on_different_versions:
				raise Exception("Two different package versions in build: %s and %s" % (built_package, self.contents[built_package.package].version))

			if vc > 0:
				debug("Choosing %s over %s", built_package, self.contents[built_package.package].version)
				self.__add_package(built_package)
			elif vc < 0:
				debug("Choosing %s over %s", self.contents[built_package.package], built_package.version)
		else:
			self.__add_package(built_package)

	def add_template(self, fn):
		t = VZTemplate(fn)
		#debug("Found template %s" % t)
		self.vz_templates.append(t)

	def add_file(self, platform, name, path):
		if not self.files.has_key(name):
			self.files[name] = []

		self.files[name].append((platform, path))
	
	def update_platforms(self):
		""" Make sure new attributes of platform go to packages """
		for platform in self.platforms:
			for package in self.contents.keys():
				if package.platform == platform:
					package.platform = platform

			for package in self.rpms.keys():
				if package.platform == platform:
					package.platform = platform

	def update_packages(self):
		""" Make sure new attributes of packages go to built packages """
		for package in self.contents.keys():
			for bp in self.contents.values():
				if bp.package == package:
					bp.package = package

	def update_rpms(self):
		""" Make sure new attributes of packages go to built packages """
		for rpm in self.rpms.keys():
			for brpm in self.rpms.values():
				if brpm.rpm == rpm:
					brpm.rpm = rpm

	def add_rpm(self, built_rpm):
		self.platforms.add(built_rpm.rpm.platform)
		if self.rpms.has_key(built_rpm.rpm) and self.config and self.config.warning.duplicates:
			warn("%s: duplicate rpm in build %s", built_rpm, self.name)
		self.rpms[built_rpm.rpm] = built_rpm
	
	def find_valid_tarball(self, package):
		if not package.content:
			return None
		if not self.tarballs_locations.has_key(package.content.filename):
			return None
		locations = self.tarballs_locations[package.content.filename]
		for i, tbd in enumerate(locations):
			if self.topdir:
				tb = os.path.join(self.topdir, tbd, package.content.filename)
			else:
				tb = os.path.join(tbd, package.content.filename)
			if i == (len(locations) - 1):
				return tb

			checksum = md5.new(open(tb, 'rb').read()).hexdigest()
			if checksum == package.content.checksum:
				return tb

		return None

	def get_news(self, other):
		""" Difference between other and this. This - considered newer """
		new_platforms = self.platforms - other.platforms
		new_packages = []
		new_versions = []
		different_contents = []
		for pkg in self.contents:
			if not other.contents.has_key(pkg):
				new_packages.append(self.contents[pkg])
			elif other.contents[pkg].version != self.contents[pkg].version:
				new_versions.append(self.contents[pkg])
			else:
				nc = self.contents[pkg].content
				oc = other.contents[pkg].content

				if nc and oc and nc.files != oc.files:
					different_contents.append(self.contents[pkg])

		return new_platforms, new_packages, new_versions, different_contents

	def get_new_rpms(self, other):
		new_rpms = []
		new_versions = []
		new_releases = []
		different_contents = []
		notes = []
		for rpm in self.rpms:
			if not other.rpms.has_key(rpm):
				notes.append("New RPM : %s" % self.rpms[rpm])
				new_rpms.append(self.rpms[rpm])
			elif other.rpms[rpm].version != self.rpms[rpm].version:
				notes.append("%s ==> %s" % (other.rpms[rpm], self.rpms[rpm]))
				new_versions.append(self.rpms[rpm])
			elif other.rpms[rpm].release != self.rpms[rpm].release:
				notes.append("%s ==> %s" % (other.rpms[rpm], self.rpms[rpm]))
				new_releases.append(self.rpms[rpm])
			else:
				nc = self.rpms[rpm].content
				oc = other.rpms[rpm].content

				if nc and oc and nc.files != oc.files:
					different_contents.append((self.rpms[rpm], describe_difference(oc.files, nc.files)))
		return new_rpms, new_versions, new_releases, different_contents, notes


__all__ = [ "Build", "BuiltPackage", "Package", "RPM", "BuiltRPM", "Platform", "is_platform_compat", "BuiltPackageContent", "compare_versions", "WindowsUpdate" ]
