import ConfigParser
import sys, socket, os, getopt
import uUtil

def usage(text, name):
	print >> sys.stderr, text % name
	sys.exit(2)

class Config:
	def __init__(self, options = [], args = [], cfg_file_opt = 'config', default_cfg = None, arglimit = (None, None), usage_txt = ""):
		optmap = {}
		short_map = {}
		short_list = ''
		long_list = []
		for opt in options:
			lopt, otype, shopt, default = opt
			if shopt:
				short_list += shopt
				short_map[shopt] = lopt
			if otype != bool:
				if shopt:
					short_list += ':'
				long_list.append(lopt + '=')
			else:
				long_list.append(lopt)
			optmap[lopt] = otype
		try:
			opts, left_args = getopt.getopt(args[1:], short_list, long_list)
		except getopt.GetoptError, e:
			print e
			usage(usage_txt, args[0])
		lmin, lmax = arglimit
		if lmin is not None and len(left_args) < lmin:
			usage(usage_txt, args[0])
		if lmax is not None and len(left_args) > lmax:
			usage(usage_txt, args[0])
		self.args = left_args

		# command line options
		for o, a in opts:
			if o[1] != '-':
				o = '--' + short_map[o[1:]]
			optname=o[2:]
			if optmap[optname] == bool:
				self.assign(optname.split(':'), True)
			else:
				self.assign(optname.split(':'), optmap[optname](a))

			if o == cfg_file_opt:
				self.read(a)
		if default_cfg:
			self.read(default_cfg)

		for opt in options:
			lopt, otype, shopt, default = opt
			self.assign(lopt.split(':'), default)

	def assign(self, what, val, override = False):
		key0 = what[0].replace('-', '')
		if len(what) == 1:
			if not self.__dict__.has_key(key0) or override:
				self.__dict__[key0] = val
			return

		if not self.__dict__.has_key(key0):
			self.__dict__[key0] = Config()
		self.__dict__[key0].assign(what[1:], val, override)

	def read(self, filename, typemapping = {}, override = False):
		parser = ConfigParser.ConfigParser()
		parser.read(filename)

		psections = parser.sections()
		for section in psections:
			if not self.__dict__.has_key(section):
				self.__dict__[section] = Config()
			options = parser.options(section)
			for option in options:
				if typemapping.has_key('%s:%s' % (section, option)):
					tp = typemapping['%s:%s' % (section, option)]
				else:
					tp = str
				self.assign([section, option], tp(parser.get(section, option)), override)

def findDBConfig():
	default_rv = {
		'dbhost' : socket.gethostname(),
		'user' : 'plesk',
		'password' : 'plesk'
	}

	if sys.platform == 'win32':
		return 	default_rv
	else:
		etc_dir = "/usr/local/pem/etc"
		kernel_conf_path = os.path.join(etc_dir, "Kernel.conf")
		odbc_ini_path = os.path.join(etc_dir, "odbc.ini")

		if not os.path.isfile(kernel_conf_path) or not os.path.isfile(odbc_ini_path):
			return default_rv

		kernel_conf = file(kernel_conf_path)
		pp = uUtil.readPropertiesFile(kernel_conf)
		kernel_conf.close()

		odbc = ConfigParser.ConfigParser()
		odbc.read(odbc_ini_path)

		dsnname = pp["dsn"]
		rv = {}
		rv['dbhost'] = odbc.get(dsnname, "servername")
		rv['user'] = pp["dsn_login"]
		rv['password'] = pp['dsn_passwd']
		return rv


dbconfig = findDBConfig()
default_logdir = uUtil.findHome()
default_logfilename = 'updater.log'

def getCommonConfigArgs():
	return [
	('database:host', str, 'h', dbconfig['dbhost']),
	('database:user', str, 'u', dbconfig['user']),
	('database:password', str, 'P', dbconfig['password']),

	('openapi:host', str, None, "localhost"), # work in only-action mode only
	('openapi:proto', str, None, 'http'),
	('openapi:port', str, None, '8440'),
	('openapi:user', str, None, None),
	('openapi:password', str, None, None),

	('log:syslog_ident', str, None, sys.argv[0]),
	('log:syslog_facility', str, 's', None),
	('log:file', str, 'l', os.path.join(default_logdir, default_logfilename)),
	('verbose', bool, 'v', False),
	('warning:duplicates', bool, None, False),
	('cache', str, 'c', None),
	('actions:onerror', str, None, None),
	('actions:instructions_ok', bool, None, False) ]

commonConfigArgs = getCommonConfigArgs()

__all__ = [ "Config", "commonConfigArgs", "getCommonConfigArgs" ]
