__rcs_id__ = """$Id: uDBBuild.py 576370 2010-05-21 10:30:38Z adedov $"""

from uBuild import *
import uDB
from uAction import *

from uLogging import *

batchSize = 1000

def saveBuildToDB(cfg, build_name, build):
	con = uDB.connect(cfg)
	cur = con.cursor()
	cur.execute("SELECT id FROM builds WHERE name=%s", build_name)
	build_id = cur.fetchone()
	if not build_id:
		raise Exception, "%s: no such build in commit-track!" % build_name
	build_id = build_id["id"]

	cur.execute("INSERT INTO build_details(build_id) VALUES(%s)", build_id)

	for platform in build.platforms:
		cur.execute("SELECT platform_id FROM platforms WHERE name=%s AND version = %s AND arch = %s", (platform.os, platform.osver, platform.arch))
		platform_id = cur.fetchone()
		if not platform_id:
			cur.execute("INSERT INTO platforms (name, version, arch) VALUES(%s, %s, %s)", platform.os, platform.osver, platform.arch)
			cur.execute("SELECT currval('platform_id_seq') AS platform_id")
			platform_id = cur.fetchone()
		platform.platform_id = platform_id["platform_id"]
	build.update_platforms()
	
	for package in build.contents.keys():
		cur.execute("SELECT package_id FROM packages WHERE name = %s AND ctype = %s AND platform_id = %s", (package.name, package.ctype, package.platform.platform_id))
		pkg_id = cur.fetchone()
		if not pkg_id:
			cur.execute("INSERT INTO packages(name, ctype, platform_id) VALUES(%s, %s, %s)",  (package.name, package.ctype, package.platform.platform_id))
			cur.execute("SELECT currval('packages_id_seq') AS package_id")
			pkg_id = cur.fetchone()
		package.package_id = pkg_id["package_id"]

	build.update_packages()

	for rpm in build.rpms.keys():
		cur.execute("SELECT rpm_id FROM rpms WHERE name = %s AND platform_id = %s", (rpm.name, rpm.platform.platform_id))
		rpm_id = cur.fetchone()
		if not rpm_id:
			cur.execute("INSERT INTO rpms(name, platform_id) VALUES(%s, %s)", (rpm.name, rpm.platform.platform_id))
			cur.execute("SELECT currval('rpms_rpm_id_seq') AS rpm_id ")
			rpm_id = cur.fetchone()
		rpm.rpm_id = rpm_id['rpm_id']
	build.update_rpms()
	
	for bp in build.contents.values():
		if bp.content:
			checksum = bp.content.checksum
			tarball = bp.content.filename
		else: 
			checksum = None
			tarball = None
		cur.execute("SELECT 1 FROM built_packages WHERE build_id = %s AND package_id = %s", (build_id, bp.package.package_id))
		if cur.fetchone():
			warn("%s: duplicate package in build", bp.package)
		else:
			cur.execute("INSERT INTO built_packages (package_id, build_id, version, tarball, checksum) VALUES (%s, %s, %s, %s, %s)", (bp.package.package_id, build_id, bp.version, tarball, checksum))
			if bp.content:
				for f in bp.content.files:
					cur.execute("INSERT INTO built_package_contents (package_id, build_id, filename, checksum) VALUES (%s, %s, %s, %s)", (bp.package.package_id, build_id, f, bp.content.files[f]))

	for brpm in build.rpms.values():
		cur.execute("SELECT 1 FROM built_rpms WHERE build_id = %s AND rpm_id = %s", (build_id, brpm.rpm.rpm_id))

		if cur.fetchone():
			warn("%s: duplicate rpm in build", brpm.rpm)
		else:
			cur.execute("INSERT INTO built_rpms(rpm_id, build_id, version, release, filename, checksum) VALUES(%s, %s, %s, %s, %s, %s)", (brpm.rpm.rpm_id, build_id, brpm.version, brpm.release, brpm.content.filename, brpm.content.checksum))
			cur.execute("SELECT currval('built_rpms_built_rpm_id_seq')")
			rpm_id = cur.fetchone()[0]
			for f in brpm.content.files:
				cur.execute("INSERT INTO built_rpm_contents (built_rpm_id, filename, checksum) VALUES(%s, %s, %s)", (rpm_id, f, brpm.content.files[f]))
	



	con.commit()

def getBuildFromDB(con, build):
	progress.do("reading %s characteristics", build)
	cur = con.cursor()
	cur.execute("SELECT id FROM builds WHERE name IN (%s, %s)", build, "build"+build)
	build_id = cur.fetchone()
	if not build_id:
		raise Exception, "%s: no such build in commit-track!" % build
	build_id = build_id["id"]

	debug("Build ID: %s" % build_id)

	progress.do("Selecting build packages")
	cur.execute("SELECT bp.package_id, bp.version, bp.tarball, bp.checksum, p.name, p.ctype, p.platform_id, o.name AS os, o.arch, o.version AS osver FROM built_packages bp JOIN packages p ON (p.package_id = bp.package_id) JOIN platforms o ON (o.platform_id = p.platform_id) WHERE bp.build_id = %s", build_id)
	progress.done()

	progress.do("Fetching build packages")
	packmap = {}
	for row in cur.fetchall():
		platform = Platform(row["os"], row["osver"], row["arch"])
		platform.platform_id = row["platform_id"]
		package = Package(row["name"], row["ctype"], platform)
		package.package_id = row["package_id"]
		packmap[package.package_id] = (package, row["version"], row["tarball"], row["checksum"])
	debug("%d packages found" % len(packmap) )
	progress.done()
	
	rv = Build(build)
	pkg_files = {}
 	progress.do("Selecting build package files")
	cur.execute("SELECT package_id, filename, checksum FROM built_package_contents WHERE build_id = %s", build_id)
	progress.done()

	progress.do("Fetching build package files")	
	cnt = 0
	rows = cur.fetchmany(batchSize)
	while len(rows) > 0:
		for row in rows:
			#row = cur.fetchone()
			#if row == None: break
			pkg_id = row['package_id']
			if pkg_id in pkg_files:
				pkg_files[pkg_id][row['filename']] = row['checksum']
			else:
				pkg_files[pkg_id] = { row['filename'] : row['checksum'] }
		rows = cur.fetchmany(batchSize)
		cnt = cnt + len(rows)
		debug("%d files" % cnt)
	progress.done()

	for p in packmap:
		files = {}
		package, version, tarball, checksum = packmap[p]
		if checksum:
			if p in pkg_files:
				files  = pkg_files[p]
			else:
				warn("Package %s has checksum, but does not have any files in content",package)
				files = {}

			content = BuiltPackageContent(tarball, checksum, files)
		else:
			content = None
		rv.add_package(BuiltPackage(package, version, content))

	progress.do("Selecting build RPMs")
	cur.execute("SELECT bp.rpm_id, bp.version, bp.release, bp.filename, bp.checksum, p.name, p.platform_id, o.name AS os, o.arch, o.version AS osver, bp.built_rpm_id FROM built_rpms bp JOIN rpms p ON (p.rpm_id = bp.rpm_id) JOIN platforms o ON (o.platform_id = p.platform_id) WHERE bp.build_id = %s", build_id)
	progress.done()

	progress.do("Fetching build RPMs")	
	rpmmap = {}
	cnt = 0
	for row in cur.fetchall():
		platform = Platform(row["os"], row["osver"], row["arch"])
		platform.platform_id = row["platform_id"]
		rpm = RPM(row["name"], platform, row["filename"])
		rpm.rpm_id = row["rpm_id"]
		rpmmap[rpm.rpm_id] = (rpm, row["version"], row["release"], row["filename"], row["checksum"], row['built_rpm_id'])
		cnt = cnt + 1
	debug("%d RPMs found" % cnt)
	progress.done()

	rpmfilesmap = {}
	progress.do("Selecting build RPM files")
	cur.execute("SELECT brc.filename AS filename, brc.checksum AS checksum, brc.built_rpm_id AS built_rpm_id FROM built_rpm_contents brc, built_rpms br WHERE brc.built_rpm_id = br.built_rpm_id AND br.build_id = %s", build_id)
	progress.done()
	progress.do("Fetching build RPM files")
	cnt = 0
	rows = cur.fetchmany(batchSize)
	while len(rows) > 0:
		for row in rows:
			rpm_id = row["built_rpm_id"]
			filename = row["filename"]
			checksum = row["checksum"]
			if not rpmfilesmap.has_key(rpm_id):
				rpmfilesmap[rpm_id] = {}
			rpmfilesmap[rpm_id][filename] = checksum
		rows = cur.fetchmany(batchSize)
		cnt = cnt + len(rows)
		debug("%d files" % cnt)
	progress.done()
	for p in rpmmap:
		rpm, version, release, filename, checksum, rpm_id = rpmmap[p]
		if rpmfilesmap.has_key(rpm_id):
			content = BuiltPackageContent(filename, checksum, rpmfilesmap[rpm_id])
		else:
			content = None
		rv.add_rpm(BuiltRPM(rpm, version, release, content))
	
	progress.done()
	return rv



__all__ = [ "saveBuildToDB", "getBuildFromDB" ]
