__rcs_id__ = """$Id: uDBFixer.py 303710 2007-08-01 14:19:05Z mkhon $"""

from uDBErrors import *
from uDBSchema import *


class DBFixer:
	# public constructor
	def __init__(self):
		self.target_con = None  # connection to target db
		self.target_tables = None  # (schema name -> schema) mapping in target db, schema names must be unique so that tables from different scs are not welcome to be fixed together

	# fix error in target db
	# @e - object derived from DBError
	def fix(e):
		if isinstance(e, DBErrorMissingTable):
			uDBSchema.createTable(e.schema, self.target_con)
		elif isinstance(e, DBErrorRedundantTable):
			uDBSchema.dropTable(e.table, self.target_con)
		elif isinstance(e, DBErrorMissingColumn):
			uDBSchema.addColumn(e.table, e.column, self.target_self.target_con)
		elif isinstance(e, DBErrorNonconformingColumnType):
			uDBSchema.changeColumnType(e.table, e.column.name, e.type, self.target_con)
		elif isinstance(e, DBErrorColumnMissingNotNull):
			uDBSchema.changeNullable(e.table, e.column, False, self.target_con)
		elif isinstance(e, DBErrorColumnRedundantNotNull):
			uDBSchema.changeNullable(e.table, e.column, True, self.target_con)
		elif isinstance(e, DBErrorColumnMissingDefault):
			uDBSchema.setDefault(e.table, e.column, e.default, self.target_con)
		elif isinstance(e, DBErrorColumnNonconformingDefault):
			uDBSchema.setDefault(e.table, e.column, e.default, self.target_con)
		elif isinstance(e, DBErrorColumnRedundantDefault):
			uDBSchema.setDefault(e.table, e.column, None, self.target_con)
		elif isinstance(e, DBErrorRedundantColumn):
			uDBSchema.dropColumn(e.table, e.column.name, self.target_con)
		elif isinstance(e, DBErrorMissingIndex):
			uDBSchema.addIndex(e.table, e.index, self.target_con)
		elif isinstance(e, DBErrorNonconformingIndex):
			uDBSchema.dropIndex(e.table, e.table_index, self.target_con)
			uDBSchema.createIndex(e.table, e.schema_index, self.target_con)
		elif isinstance(e, DBErrorRedundantIndex):
			uDBSchema.dropIndex(e.table, e.table_index, self.target_con)
		elif isinstance(e, DBErrorMissingPrimaryKey):
			DBSchema.addConstraint(e.table, e.schema_pk, self.target_con)
		elif isinstance(e, DBErrorNonconformingPrimaryKey):
			# - determining tables referencing to this pk
			# - if found then save and drop those fk constraints
			if self.target_tables is not None:
				table_fks = {}  # (table name -> fk set) mapping in target db
				for table in self.target_tables.values():
					for constraint in table.constraints:
						if isinstance(constraint, ForeignKey) and constraint.reftable == e.table.name and constraint.columns.sort() == e.table_pk.columns.sort():
							if table_fks.has_key(constraint.name):
								table_fks[constraint.name] = []
							table_fks[constraint.name] += constraint
							table.dropConstraint(constraint, self.target_con)

			uDBSchema.dropConstraint(e.table, e.table_pk, self.target_con)
			uDBSchema.addConstraint(e.table, e.schema_pk, self.target_con)

			# - restoring dropped fk constraints
			if self.target_tables is not None:
				for table_name in table_fks.keys():
					for fk in table_fks[table_name]:
						self.target_tables[table_name].addConstraint(fk)
		elif isinstance(e, DBErrorRedundantPrimaryKey):
			uDBSchema.dropConstraint(e.table, e.table_pk, self.target_con)
		elif isinstance(e, DBErrorMissingUnique):
			uDBSchema.addConstraint(e.table, e.schema_unique, self.target_con)
		elif isinstance(e, DBErrorNonconformingUnique):
			uDBSchema.dropConstraint(e.table, e.table_unique, self.target_con)
			uDBSchema.addConstraint(e.table, e.schema_unique, self.target_con)
		elif isinstance(e, DBErrorRedundantUnique):
			uDBSchema.dropConstraint(e.table, e.table_unique, self.target_con)
		elif isinstance(e, DBErrorMissingForeignKey):
			uDBSchema.addConstraint(e.table, e.schema_fk, self.target_con)
		elif isinstance(e, DBErrorNonconformingForeignKey):
			uDBSchema.dropConstraint(e.table, e.table_fk, self.target_con)
			uDBSchema.addConstraint(e.table, e.schema_fk, self.target_con)
		elif isinstance(e, DBErrorRedundantForeignKey):
			uDBSchema.dropConstraint(e.table, e.table_fk, self.target_con)
		elif isinstance(e, DBErrorMissingCheck):
			uDBSchema.addConstraint(e.table, e.schema_check, self.target_con)
		elif isinstance(e, DBErrorNonconformingCheck):
			uDBSchema.dropConstraint(e.table, e.table_check, self.target_con)
			uDBSchema.addConstraint(e.table, e.schema_check, self.target_con)
		elif isinstance(e, DBErrorRedundantCheck):
			uDBSchema.dropConstraint(e.table, e.check_fk, self.target_con)
		elif isinstance(e, DBErrorMissingSequence):
			pass
		elif isinstance(e, DBErrorRedundantSequence):
			pass
		elif isinstance(e, DBErrorRedundantView):
			pass
		elif isinstance(e, DBErrorRedundantTrigger):
			pass
		elif isinstance(e, DBErrorNonconformingFunction):
			pass
		elif isinstance(e, DBErrorRedundantFunction):
			pass
		elif isinstance(e, DBError):
			pass

