__rcs_id__ = """$Id: uDBSchemaPgSQL.py 428566 2009-07-30 03:06:23Z csvintsoff $"""
from uDBSchema import *
import uDBTypes
import uPostgreSQLTypes
from uGenerics import first
from uLogging import err

""" uDBSchemaPgSQL  module should never be imported directly. it is implementation of PgSQL specific part of uDBSchema module """

def getColumnType(attypid, attlen, attypmod):
	if attypid in uPostgreSQLTypes.char_types:
		if attypid ==  uPostgreSQLTypes.PG_TYPE_BPCHAR:
			return uDBTypes.CharType(attypmod - 4)
		else:
			return uDBTypes.CharType(attlen)
	elif attypid == uPostgreSQLTypes.PG_TYPE_VARCHAR:
		return uDBTypes.VarcharType(attypmod - 4)
        elif attypid in uPostgreSQLTypes.blob_types:
                return uDBTypes.BlobType()
	elif attypid == uPostgreSQLTypes.PG_TYPE_TEXT:
		return uDBTypes.VarcharType(attlen)
	elif attypid in uPostgreSQLTypes.int_types:
		return uDBTypes.IntType()
	elif attypid == uPostgreSQLTypes.PG_TYPE_INT8:
		return uDBTypes.BigIntType()
	elif attypid in uPostgreSQLTypes.timestamp_types:
		return uDBTypes.TimestampType()
	else:
		return uDBTypes.Type(attypid, attlen, attypmod)

public_ns = None

def get_public_ns(cur):
	global public_ns
	if public_ns is not None:
		return public_ns
	cur.execute("SELECT oid::int FROM pg_namespace WHERE nspname = 'public'")
	public_ns = cur.fetchone()[0]
	return public_ns
	

def getTable(name, con):
	cur = con.cursor()
	columns = []
	public_ns = get_public_ns(cur)
	cur.execute("SELECT oid::int FROM pg_class WHERE relname = %s AND relnamespace = %s", name, public_ns)
	row = cur.fetchone()
	if not row:
		raise NoSuchTable(name)
	oid = row[0]

	cur.execute("SELECT attname, atttypid::int, attlen, atttypmod, attnotnull,attnum, atthasdef FROM pg_attribute WHERE attrelid = %s AND attnum > 0 AND NOT attisdropped ORDER by attnum", oid)

	rows = cur.fetchall()
	for row in rows:
		coltype = getColumnType(row[1], row[2], row[3])
		if row[6]:
			cur.execute("SELECT adsrc FROM pg_attrdef WHERE adrelid = %s AND adnum = %s", (oid, row[5]))
			default = cur.fetchone()[0]
			if default.startswith('nextval('):
				# not very correct, but correct for
				# PEM created tables.
				default = None
				coltype = uDBTypes.SerialType()
			elif default == 'now()':
				default = uDBTypes.DefSysDate()

		else:
			default = None
		columns.append(Column(row[0], coltype, not row[4], default, row[5]))
	cur.execute("SELECT conname, contype::text, confrelid::int, confupdtype::text, confdeltype::text, array_to_string(conkey, ' '), array_to_string(confkey, ' '), consrc FROM pg_constraint WHERE conrelid = %s", oid)
	rows = cur.fetchall()
	constraints = []
	for row in rows:
		contype = row[1]
		if contype in ('c', 'p', 'u', 'f'):
			constr_columns = []
			for num in row[5].split():
				num = int(num)
				constr_columns += [ x.name for x in columns if x.num == num]
			if contype == 'p':
				constraints.append(PrimaryKey(row[0], constr_columns))
			elif contype == 'u':
				constraints.append(UniqueKey(row[0], constr_columns))
			elif contype == 'c':
				constraints.append(Check(row[0], row[7]))
			else:
				cur.execute("SELECT attname FROM pg_attribute WHERE attrelid = %s AND attnum > 0 ORDER BY attnum", row[2])
				foreign_table_columns = [ x[0] for x in cur.fetchall()]

				fk_columns = [ foreign_table_columns[int(x)-1] for x in row[6].split()]
				cur.execute("SELECT relname FROM pg_class WHERE oid = %s AND relnamespace = %s", row[2], public_ns)
				ft_name = cur.fetchone()[0]
				onupdate = act_mapping[row[3]]
				ondelete = act_mapping[row[4]]

				constraints.append(ForeignKey(row[0], ft_name, (constr_columns, fk_columns), ondelete, onupdate))
	cur.execute("SELECT array_to_string(indkey, ' '), indexrelid::int FROM pg_index WHERE indrelid = %s", oid)
	rows = cur.fetchall()
	indexes = []
	for row in rows:
		cur.execute("SELECT relname FROM pg_class WHERE oid = %s AND relnamespace = %s", row[1], public_ns)
		indname = cur.fetchone()[0]
		same_constr = [ x for x in constraints if x.name == indname]
		if same_constr:
			continue
		ind_columns = []
		for num in row[0].split():
			ind_columns += [ x.name for x in columns if x.num == int(num)]
		indexes.append(Index(indname, ind_columns))

	cur.close()
	return Table(name, columns, constraints, indexes)

act_mapping = {
	'a' : "NO ACTION",
	'c' : "CASCADE",
	'r' : "RESTRICT",
	'n' : "SET NULL"

};

def dropConstraint(tab, name, con):
	cur = con.cursor()
	cur.execute("ALTER TABLE %s DROP CONSTRAINT %s CASCADE" % (tab.name, name))


def addConstraint(tab, constr, con):
	cur = con.cursor()
	try :
		cur.execute(constr.getStatement(tab.name))
	finally:
		cur.close()
	

def createTable(tab, con):
	cur = con.cursor()
	for c in  [ x for x in tab.columns if isinstance(x.type, uDBTypes.SerialType)]:
		cur.execute("CREATE SEQUENCE %s_seq START WITH %s" % (tab.name, c.type.startswith));
		
	stmt = "CREATE TABLE %s (%s) WITH OIDS" % (tab.name, ',\n'.join([x.descr(tab) for x in tab.columns + tab.constraints]))

	cur.execute(stmt)
	cur.close()

def dropTable(tab, con):
	cur = con.cursor()
	cur.execute("DROP TABLE %s" % tab.name)
	if [ x for x in tab.columns if isinstance(x.type, uDBTypes.SerialType)]:
		cur.execute("DROP SEQUENCE %s_seq" % tab.name);

	cur.close()

def changeColumnType(tab, name, new_type, con):
	cur = con.cursor()
	cur.execute("ALTER TABLE %s ALTER %s TYPE %s" % (tab.name, name, new_type.PgSQL_name()))
	cur.close()

def dropColumn(tab, name, con):
	cur = con.cursor()
	cur.execute("ALTER TABLE %s DROP COLUMN %s" % (tab.name, name))
	for x in tab.columns:
		if x.name == name and isinstance(x, uDBTypes.SerialType):
			cur.execute("DROP SEQUENCE %s_seq" % tab.name)
	cur.close()

def _defaultDesc(defval):
	if defval is None:
		return ''
	elif isinstance(defval, uDBTypes.DefSysDate):
		return 'DEFAULT now()'
	else:
		return 'DEFAULT %s' % defval

def addColumn(tab, column, con):
	cur = con.cursor()
	default_str = _defaultDesc(column.default)
	if isinstance(column.type, uDBTypes.SerialType):
		cur.execute("CREATE SEQUENCE %s_seq" % tab.name);
		default_str = "DEFAULT nextval('%s_seq')" % tab.name
	cur.execute("ALTER TABLE %s ADD %s %s %s %s" % (tab.name, column.name, column.type.PgSQL_name(), not column.nullable and 'NOT NULL' or '', default_str))


def renameTable(tab, new_name, con):
	cur = con.cursor()
	cur.execute("ALTER TABLE %s RENAME to %s" % (tab.name, new_name))

	for col in tab.columns:
		if isinstance(col.type, uDBTypes.SerialType):
			cur.execute("ALTER TABLE %s_seq RENAME TO %s_seq" % (tab.name, new_name))
			cur.execute("ALTER TABLE %s ALTER %s SET DEFAULT nextval('%s_seq')" % (new_name, col.name, new_name))



def changeNullable(table, column, notnullable, con):
	cur = con.cursor()
	cur.execute("ALTER TABLE %s ALTER %s %s NOT NULL" % (table.name, column, notnullable and 'SET' or 'DROP'))
	cur.close()

def setDefault(table, column, default, con):
	cur = con.cursor()
	try :
		if default is None: 
			cur.execute("ALTER TABLE %s ALTER %s DROP DEFAULT" % (table.name, column))
		elif isinstance(default, uDBTypes.DefSysDate):
			cur.execute("ALTER TABLE %s ALTER %s SET DEFAULT now()" % (table.name, column))
		else:
			cur.execute("ALTER TABLE %s ALTER %s SET DEFAULT %s" % (table.name, column, default))

	finally:
		cur.close()


def dropIndex(dummy, name, con):
	cur = con.cursor()
	cur.execute("DROP INDEX %s" % name)

def createIndex(table, index, con):
	cur = con.cursor()
	stmt = "CREATE INDEX %s ON %s(%s)" % (index.name, table.name, ', '.join(index.columns))
	cur.execute(stmt)


def renameColumn(table, old_name, new_name, con):
	cur = con.cursor()
	stmt = "ALTER TABLE %s RENAME %s TO %s" % (table.name, old_name, new_name)
	cur.execute(stmt)

def columnDescription(col, table):
	props = { "name" : col.name, "null": not col.nullable and "NOT NULL" or "", "type": col.type.PgSQL_name() }
	if col.default is not None and isinstance(col.default, uDBTypes.DefSysDate):
		props['def'] = 'DEFAULT now()'
	elif isinstance(col.type, uDBTypes.SerialType):
		props['def'] = "DEFAULT nextval('%s_seq')" % table.name
	elif col.default is not None:
		props['def'] = "DEFAULT %s" % col.default
	else:
		props['def'] = ''

	return "%(name)s %(type)s %(def)s %(null)s" % props


def identityInsertBegin(dummy, dummy2):
	pass

def identityInsertEnd(table, con):
	for column in table.columns:
		if isinstance(column.type, uDBTypes.SerialType):
			cur = con.cursor()
			cur.execute("SELECT MAX(%s) FROM %s" % (column.name, table.name))
			row = cur.fetchone()
			if row and row[0] and row[0] > 0:
				cur.execute(("SELECT setval('%s_seq', %%s)" % table.name), row[0])
			cur.close()

def dropIdentity(tab, con):
	identity_column = first(tab.columns, lambda x: isinstance(x.type, uDBTypes.SerialType))
	if identity_column is None:
		err("There is no identity column on table %s", tab.name)
		return

	cur = con.cursor()
	cur.execute("ALTER TABLE %s ALTER %s DROP DEFAULT" % (tab.name, identity_column.name))
	cur.execute("DROP SEQUENCE %s_seq" % tab.name)



def getTableList(con):
	cur = con.cursor()
	public_ns = get_public_ns(cur)
	cur.execute("SELECT relname FROM pg_class WHERE relkind = 'r' AND relname NOT like 'pg_%%' AND relname != 'dual' AND relnamespace = %s", public_ns)
	return [row[0] for row in cur.fetchall()]

def exprEq(dummy1, dummy2):
	# TODO
	return True

def keyDescription(key):
	return "CONSTRAINT %s %s (%s)" % (key.name, key.kind(), ', '.join(key.columns))

def fkDescription(fk):
	return "CONSTRAINT %s FOREIGN KEY (%s) REFERENCES %s(%s) ON DELETE %s" % (fk.name, ', '.join(fk.columns[0]), fk.reftable, ', '.join(fk.columns[1]), fk.ondelete)
__all__ = [ 'getTable' ]
