#!/usr/bin/python

MN_pkg = ('chief', 'sc')
UI_pkg = ('chief', 'cp')

class NativePackage:
	def __init__(self, name, nptype, host, where, where_vztempl=None, name_x64 = None):
		self.name = name # Filename
		self.name_x64 = name_x64 or name
		self.type = nptype or "rpm"
		self.host = host
		self.where = where
		self.where_vztempl = where_vztempl
		self.package_name = None

class PkgInstallation:
	def _node_init__(self, node):
		self.name	= node.getAttribute("name")
		self.ctype	= node.getAttribute("ctype")
		where = node.getAttribute("host")
		where_pkg_name = node.getAttribute("where_pkg_name")
		if where == "where_pkg":
			self.where_pkg = (where_pkg_name, node.getAttribute("where_pkg_ctype") or "service")
		elif where == "mn":
			self.where_pkg = MN_pkg
		elif where == "ui":
			self.where_pkg = UI_pkg
		elif where == "no":
			self.where_pkg = None
		elif where_pkg_name:
			self.where_pkg = (where_pkg_name, node.getAttribute("where_pkg_ctype") or "service")
		else: #upgrade
			self.where_pkg = (self.name, self.ctype)

	def _pkg_init__(self, name, ctype, where):
		self.name	= name
		self.ctype	= ctype
		self.where_pkg	= where

	def __init__(self, node=None, name=None, ctype=None, where=None):
		if node:
			PkgInstallation._node_init__(self, node)
		else:
			PkgInstallation._pkg_init__(self, name, ctype, where)

	def node(self, doc):
		rv = doc.createElement("PACKAGE")
		rv.setAttribute("name", self.name)
		rv.setAttribute("ctype", self.ctype)
		if self.where_pkg == ('chief', 'sc'):
			rv.setAttribute("host", "mn")
		elif self.where_pkg == ('chief', 'cp'):
			rv.setAttribute("host", "ui")
		elif self.where_pkg == (self.name, self.ctype):
			rv.setAttribute("host", "upgrade")
		elif self.where_pkg:
			nm, ctype = self.where_pkg
			rv.setAttribute("host", "where_pkg")
			rv.setAttribute("where_pkg_name", nm)
			rv.setAttribute("where_pkg_ctype", ctype)
		else:
			rv.setAttribute("host", "no")

		return rv

def parseNativePackageNode(node):
	host = node.getAttribute("host") or "all"
	name = node.getAttribute("name")
	name_x64 = node.getAttribute("name_x64") or name
	ptype = node.getAttribute("type") or "rpm"

	where_pkg_name = node.getAttribute("where_pkg_name")
	if where_pkg_name:
		where = (node.getAttribute("where_pkg_ctype") or "service", where_pkg_name)
	else:
		where = None # None will mean everywhere. 
	where_templ = node.getAttribute("where_vztemplate")
	if not where_templ:
		where_templ = None
	return NativePackage(name, ptype, host, where, where_templ, name_x64)

__all__ = [ "PkgInstallation", "NativePackage", "MN_pkg", "UI_pkg", "parseNativePackageNode" ]
