from __future__ import generators

__rcs_id__ = """$Id: uDLReader.py 714565 2011-11-25 08:25:39Z alivanov $"""

__pychecker__ = "blacklist=modules/sets.py unusednames=__rcs_id__,dummy"

from xml.dom import minidom as dom
from uAction import *
import uActionContext
from uLogging import *
from uPEM import *
import uUtil
import sets
import tempfile
import uSysDB, os, sys
import uDialog
import gc

from uManifestParser import nodeTextContent
from uDLCommon import *


class Action:
	def __init__(self, id):
		self.id = id
	
	def kind(self):
		return "atomic action"


class Script(Action):
	def __init__(self, node, current_update):
		Action.__init__(self, node.getAttribute("id"))
		self.code = ''
		if node.hasAttribute("dbowner"):
			self.dbowner = node.getAttribute("dbowner")
		elif node.hasAttribute("owner"):
			self.dbowner = node.getAttribute("owner")
		else:
			self.dbowner = "plesk"

		self.installation = node.getAttribute("installation")

		if node.hasAttribute("script"):
			upddir = os.path.dirname(current_update)
			self.code += file(os.path.join(upddir, node.getAttribute("script"))).read().strip() + '\n'

		self.code += nodeTextContent(node).strip()+'\n'
		self.update_file = None

	def build_node(self, doc, name):
		rv = doc.createElement(name)
		rv.setAttribute("id", self.id)
		rv.setAttribute("dbowner", self.dbowner)
		if self.installation:
			rv.setAttribute("installation", self.installation)
		codenode = doc.createCDATASection(self.code)
		rv.appendChild(codenode)
		return rv

from cStringIO import StringIO


def stmtKind(stmt):
	norm = stmt.upper().strip()
	for kind in ['BEGIN', 'COMMIT', 'ROLLBACK', 'SELECT', 'CREATE', 'ALTER', 'DROP']:
		if norm.startswith(kind):
			return kind
	else:
		return ''

class SQLScript(Script):
	def __init__(self, node, current_update):
		Script.__init__(self, node, current_update)

	def node(self, doc):
		return self.build_node(doc, "SQL")

	def kind(self):
		return "SQL"

	def parsed_code(self):
		return [ (stmtKind(stmt), stmt) for stmt in uUtil.stmt_parser(self.code, uSysDB.ConcatOperator, uSysDB.nowfun) ]
	
	def execute(self):
		dbname = uSysDB.sc_database(self.dbowner)
		if not dbname:
			notice("%s is not installed, skipping", self.dbowner)
			return None

		if self.installation and self.installation != installation_id():
			notice("%s should not be performed here", self.id)
			return None

		con = uSysDB.connect(self.dbowner)
		cur = con.cursor()
		rv = None
		for stmt in uUtil.stmt_parser(self.code, uSysDB.ConcatOperator, uSysDB.nowfun):
			debug('executing %s', stmt)
			kind = stmtKind(stmt)
			if kind in ('BEGIN', 'COMMIT', 'ROLLBACK'):
				warn('%s statements are ignored, <SQL> action always implicitely begins and commits transaction', kind)
			elif kind == 'SELECT':
				cur.execute(stmt)
				rv = tuple([ col for col in cur.fetchone() ])
			else:
				cur.execute(stmt)
		con.commit()
		con.close(); del con
		
		return rv

class ExecScript(Script):
	def __init__(self, node, current_update):
		Script.__init__(self, node, current_update)


	def node(self, doc):
		return self.build_node(doc, "EXEC")
	
	def makeFunctionCode(self, funname):
		lines = self.code.strip().splitlines()
		function_code = "def %s():\n" % funname
		inquote = 0
		for line in lines:
			if not inquote:
				function_code += '\t'

			function_code += line
			function_code += '\n'

			num_quotes = line.count('"""') + inquote

			inquote = divmod(num_quotes,2)[1]

		return function_code
		
	def execute(self):
		if not uSysDB.sc_database(self.dbowner):
			notice("%s is not installed, skipping", self.dbowner)
			return None

		if self.installation and self.installation != installation_id():
			notice("%s should not be performed here", self.id)
			return None

		curdir = os.getcwd()
		uActionContext.prepare(self.id)
		dictionary = {}

		function_code = self.makeFunctionCode("ActionFun")

		dirname = ''
		if self.update_file:
			dirname = os.path.dirname(self.update_file)
			if dirname:
				os.chdir(dirname)
		try:
			try:
				exec function_code in dictionary
				result = eval('ActionFun()', dictionary)
				uActionContext.commit()
				return result
			except:
				uSysDB.rollback_all()
				uActionContext.rollback()
				raise
		finally:
			os.chdir(curdir)
			debug("Garbage collecting: %s", gc.collect())
	
def get_instruction_clause(node):
	return (nodeTextContent(node).strip(),[ get_instruction_clause(child) for child in node.childNodes if child.nodeType == dom.Node.ELEMENT_NODE and child.tagName == u'CLAUSE'] )

def append_sub_instructions(instructions, node, doc):
	for instruction in instructions:
		if type(instruction) == str:
			node.appendChild(doc.createCDATASection(instruction))
		elif type(instruction) == list:
			for elem in instruction:
				sub_node = doc.createElement("CLAUSE")
				append_sub_instructions(elem, sub_node, doc)
				node.appendChild(sub_node)
		elif type(instruction) == tuple:
			top, sub = instruction
			sub_node = doc.createElement("CLAUSE")
			node.appendChild(doc.createCDATASection(top))
			append_sub_instructions(sub, sub_node, doc)
		else:
			print instruction
			err("Unknown type of instruction: %s", type(instruction))
		
class Instruction(Action):
	def __init__(self, node):
		Action.__init__(self, node.getAttribute("id") or None)
		if node.hasAttribute("dbowner"):
			self.dbowner = node.getAttribute("dbowner")
		else:
			self.dbowner = "plesk"
		self.interactive = node.getAttribute("interactive") == 'yes'
		self.content = get_instruction_clause(node)
	
	def kind(self):
		return "instruction"

	def node(self, doc):
		rv = doc.createElement("INSTRUCTION")
		if self.id:
			rv.setAttribute("id", self.id)
		if self.interactive:
			rv.setAttribute("interactive", "yes")
		if self.dbowner != 'plesk':
			rv.setAttribute("dbowner", self.dbowner)
		append_sub_instructions(self.content, rv, doc)
		return rv

	def execute(self):
		if not self.interactive:
			return None
		if not uSysDB.sc_database(self.dbowner):
			notice("%s is not installed, skipping", self.dbowner)
			return None
		print "=============================================================================="
		print_instruction_content(self.content, sys.stdout, '')
		print "=============================================================================="
		print 'Is everything ok?'
		if _config and _config.actions.instructions_ok:
			return None
		if not uDialog.askYesNo():
			raise Exception, "Interrupted by user"
		return None

def print_instruction_content( content, file, prefix):
	instruction, sub_instructions = content
	if instruction:
		print >> file, '\n%s %s' % (prefix, instruction)

	if sub_instructions:
		for index in xrange(len(sub_instructions)):
			p2 = '%s%d.' % (prefix, index+1)
			print_instruction_content(sub_instructions[index], file, p2)

class UpdBase:
	def __init__(self):
		self.actions = []
		self.required = None
		self.kind = None

	def add_action(self, action):
		action.update_file = self.current_update
		self.actions.append(action)

	def add_exec(self, node, current_update):
		self.add_action(ExecScript(node, current_update))
	
	def add_sql(self, node, current_update):
		self.add_action(SQLScript(node, current_update))
	
	def add_instruction(self, node, dummy):
		self.actions.append(Instruction(node))

	def process(self, node, current_update = None):
		self.current_update = current_update
		self.name = node.getAttribute("name")
		self.version = node.getAttribute("version")
		if self.required is None:
			self.required = node.getAttribute("required")
		self.built = node.getAttribute("built")
		self.kind = node.getAttribute("type") or self.kind or "release"
		if not self.built:
			import time
			self.built = time.asctime()
		for child in [ x for x in node.childNodes if x.nodeType == dom.Node.ELEMENT_NODE ]:
			if XML_mapping.has_key(child.tagName):
				XML_mapping[child.tagName](self, child, current_update)
			else:
				raise Exception, "%s: unknown action" % child.tagName
		self.current_update = None

		return self.name, self.version, self.built, self.kind

	def add_if(self, node, dummy=None):
		self.actions.append(ConditionalUpdate(node))

	def save_actions(self, doc, node):
		for act in self.actions:
			node.appendChild(act.node(doc))

class Condition:
	def __init__(self, node):
		if node.name == "INSTALLATION":
			self.set_identity = node.getAttribute("identity")
			self.pkg = None
		elif node.name == "PKG_INSTALLED":
			self.pkg = (node.getAttribute("name"), node.getAttribute("ctype"))
			self.set_identity = None
		else:
			raise Exception, "%s: unknown" % node.name
	
	def node(self, doc):
		if self.pkg:
			nm, ctype = self.pkg
			rv = doc.createElement("PKG_INSTALLED")
			rv.setAttribute("name", nm)
			rv.setAttribute("ctype", ctype)
		else:
			rv = doc.createElement("INSTALLATION")
			rv.setAttribute("identity", self.set_identity)

		return rv

def savePackages(node, doc, packages):
	to_add = []
	for package in packages:
		for install_on in packages[package]:
			to_add.append((package[1], PkgInstallation(None, package[0], package[1], install_on).node(doc)))

	to_add.sort()

	for np in to_add:
		node.appendChild(np[1])

class ConditionalUpdate(UpdBase):
	def __init__(self, node):
		UpdBase.__init__(self)
		self.packages = {}
		for child in [ x for x in node.childNodes if node.nodeType == dom.Node.ELEMENT_NODE ]:
			if child.name in ("INSTALLATION", "PKG_INSTALLED"):
				self.condition = Condition(child)
			elif XML_mapping.has_key(child.name):
				XML_mapping(child.name)(self, child)
			else:
				raise Exception, "%s: unknown action", child.name
	def node(self, doc):
		rv = doc.createElement("IF")
		rv.appendChild(self.condition.node(doc))
		self.save_actions(doc, rv)
		savePackages(rv, doc, self.packages)
		return rv


doAlwaysPre = [
('Ensure that all PPM mirrors are created on management node, or accessible via ssh from management node, by names known by system (you can get those names using "SELECT host_id FROM ppm_mirrors" on system database (plesk)', None),
("""Prepare for upgrade (this step does not require system downtime, and thus can be performed any time before starting real upgrade):
./update --only-prepare --cache path_to_directory_for_cache [--skip:vacuum] [--database:host name_of_sysdb_host] [--database:user username to connect to system database] [--database:password password for user to connect to system database]
--skip:vacuum updater will not perform database vacuuming;
--database:user is useful for installation on Windows, if you specify non-default username (not a plesk), --database:password is useful for Windows too.
""", None)
]
doAlways = [
("""Execute included update script (it located just at the root of distribution) on management node:
./update --cache path_to_directory_for_cache [--skip:vacuum] [--database:host name_of_sysdb_host] [--database:user username to connect to system database] [--database:password password for user to connect to system database]
--skip:vacuum updater will not perform database vacuuming.
If you wish to install several hotfixes at once, you can execute following command instead (it will be faster than installing hotfixes one by one):
latest_hotfix/update --cache cache [--database:host hostname] [--database:user username] [--database:password] hotfix_1_directory hotfix_2_directory ... latest_hotfix
""", None)]

	
class Update(UpdBase):
	def __init__(self):
		UpdBase.__init__(self)


		self.prepreparation = UpdBase()
		self.preparation = UpdBase()

		self.conflicts = []

		self.pre = UpdBase()
		self.prepkg = UpdBase()
		self.post = UpdBase()
		self.packages = {}
		self.build_scripts = []
		self.to_copy = []
		self.current_update = None
		self.disabled_packages = sets.Set()

		self.native_packages = []

		self.update_windows = False

		self.cleanup = UpdBase()

		self.kind = "release"

	def add_build_exec(self, node, dummy):
		self.build_scripts.append(nodeTextContent(node))
	
	def add_copy(self, node, dummy):
		self.to_copy.append((node.getAttribute("path"), node.getAttribute("to")))

	def add_post(self, node, dummy):
		self.post.process(node, self.current_update)

	def add_prepkg(self, node, dummy):
		self.prepkg.process(node, self.current_update)

	def add_pre(self, node, dummy):
		self.pre.process(node, self.current_update)
	
	def add_prepare(self, node, dummy):
		self.preparation.process(node, self.current_update)

	def add_preprepare(self, node, dummy):
		self.prepreparation.process(node, self.current_update)
	
	def add_cleanup(self, node, dummy):
		self.cleanup.process(node, self.current_update)

	def has_package(self, name, ctype):
		return self.packages.has_key((name, ctype))

	def add_package(self, pkg):
		if self.packages.has_key((pkg.name, pkg.ctype)):

			self.packages[(pkg.name, pkg.ctype)].update([pkg.where_pkg])
		else:
			self.packages[(pkg.name, pkg.ctype)] = sets.Set([pkg.where_pkg])
	def add_node_package(self, node, dummy):
		if node.getAttribute('host') == 'no_auto':
			self.disabled_packages.add((node.getAttribute('name'), node.getAttribute('ctype')))
		else:
			self.add_package(PkgInstallation(node))
	
	def add_new_package(self, name, ctype):
		self.add_package(PkgInstallation(None, name, ctype, None))
	
	def add_upgrade_package(self, name, ctype):
		self.add_package(PkgInstallation(None, name, ctype, (name, ctype)))
	def add_native_package(self, node, dummy):
		pkg = parseNativePackageNode(node)
		self.native_packages.append(pkg)
		if pkg.name == "install_win_sn.exe":
			self.update_windows = True

	def add_conflict(self, node, dummy):
		self.conflicts.append((self.name, node.getAttribute("name")))

	def save_native_packages(self, doc, node):
		for pkg in self.native_packages:
			elem = doc.createElement("NATIVE_PACKAGE")
			elem.setAttribute("name", pkg.name)
			elem.setAttribute("name_x64", pkg.name_x64)
			elem.setAttribute("host", pkg.host)
			elem.setAttribute("type", pkg.type)
			if pkg.where is not None:
				typ, nam = pkg.where
				elem.setAttribute("where_pkg_ctype", typ)
				elem.setAttribute("where_pkg_name", nam)
			if pkg.where_vztempl is not None:
				elem.setAttribute("where_vztemplate", pkg.where_vztempl)

			node.appendChild(elem)

	
	def save_instructions(self, fn, prefix=''):
		f = file(fn, "w")
		print >> f, prefix
		print >> f, """
================= Preparation to update ===============
Following steps do not require control panel downtime, but may take considerable time (time depends on amount of hardware in system). It is recommended to execute them in advance.
"""
		all_instructions = [ x.content for x in self.prepreparation.actions if hasattr(x, 'content') and hasattr(x, 'interactive') and not x.interactive ]
		all_instructions += doAlwaysPre
		all_instructions += [ x.content for x in self.preparation.actions if hasattr(x, 'content') and hasattr(x, 'interactive') and not x.interactive ]
		print_instruction_content((None, all_instructions), f, '')
		print >> f, """
===================== Update itself ===================
During following steps, control panel will be down. It is recommended to stop watchdog before proceeding further, to get rid of false notifications
"""

		all_instructions = [ x.content for x in self.pre.actions if hasattr(x, 'content') and hasattr(x, 'interactive') and not x.interactive ]

		all_instructions += doAlways
		bad_instructions = [ x.content for x in self.actions if hasattr(x, 'content') and hasattr(x, 'interactive') and not x.interactive ]
		if bad_instructions:
			print >> sys.stderr, "Non interactive instructions in database upgrade phase:"
			print_instruction_content((None, bad_instructions), sys.stderr, '')
			raise Exception("Non-interactive instructions should be always in 'PRE', 'PREPARE', 'POST' or 'CLEANUP'")

		all_instructions += [ x.content for x in self.post.actions if hasattr(x, 'content') and hasattr(x, 'interactive') and not x.interactive ]
		print_instruction_content((None, all_instructions), f, '')

		all_instructions = [ x.content for x in self.cleanup.actions if hasattr(x, 'content') and hasattr(x, 'interactive') and not x.interactive ]
		if all_instructions:
			print >> f, """
===================== Cleaning up ====================
Following steps will not affect system functionality, do not require downtime and are optional. You may perform them any time after upgrade is complete.
"""
			print_instruction_content((None, all_instructions), f, '')


	def save_disabled_packages(self, doc, node):
		for name, ctype in self.disabled_packages:
			elem = doc.createElement("PACKAGE")
			elem.setAttribute("name", name)
			elem.setAttribute("host", "no_auto")
			elem.setAttribute("ctype", ctype)

			node.appendChild(elem)

	def save_conflicts(self,doc):
		for dummy, c in self.conflicts:
			cnode = doc.createElement("CONFLICTS")
			cnode.setAttribute("name", c)
			doc.documentElement.appendChild(cnode)


	def save(self, to):
		impl = dom.getDOMImplementation()
		dt = impl.createDocumentType("UDL2", "UDL2", "UDL2")
		doc = impl.createDocument("UDL2", "UDL2", dt)
		root = doc.documentElement
		root.setAttribute("name", self.name)
		root.setAttribute("version", self.version)
		root.setAttribute("required", self.required)
		root.setAttribute("built", self.built)
		root.setAttribute("type", self.kind)
		self.save_conflicts(doc)
		self.save_native_packages(doc, root)

		prepreparation_node = doc.createElement("PREPREPARE")
		self.prepreparation.save_actions(doc, prepreparation_node)
		root.appendChild(prepreparation_node)

		preparation_node = doc.createElement("PREPARE")
		self.preparation.save_actions(doc, preparation_node)
		root.appendChild(preparation_node)

		pre_node = doc.createElement("PRE")
		self.pre.save_actions(doc, pre_node)
		root.appendChild(pre_node)

		savePackages(root, doc, self.packages)
		self.save_disabled_packages(doc, root)
		self.save_actions(doc, root)

		prepkg_node = doc.createElement("PRE_PKG")
		self.prepkg.save_actions(doc, prepkg_node)
		root.appendChild(prepkg_node)

		post_node = doc.createElement("POST")
		self.post.save_actions(doc, post_node)
		root.appendChild(post_node)

		cleanup_node = doc.createElement("CLEANUP")
		self.cleanup.save_actions(doc, cleanup_node)
		root.appendChild(cleanup_node)

		doc.writexml(writer=file(to, "w+"), indent = "\t", addindent="\t", newl="\n")


XML_mapping = {
	"CONFLICTS"	: Update.add_conflict,
	"PREPREPARE"	: Update.add_preprepare,
	"PREPARE"	: Update.add_prepare,
	"PRE"		: Update.add_pre,
	"PRE_PKG"	: Update.add_prepkg,
	"POST"		: Update.add_post,
	"CLEANUP"	: Update.add_cleanup,
	"EXEC"		: UpdBase.add_exec,
	"SQL"		: UpdBase.add_sql,
	"INSTRUCTION"	: UpdBase.add_instruction,
	"BUILD_EXEC"	: Update.add_build_exec,
	"IF"		: UpdBase.add_if,
	"COPY"		: Update.add_copy,
	"PACKAGE"	: Update.add_node_package,
	"NATIVE_PACKAGE": Update.add_native_package
}


def depends(what, on, depend_map):
	counter = 0
	max_counter = len(depend_map) + 1

	cur = what
	while counter < max_counter:
		counter += 1
		if not depend_map.has_key(cur):
			notice("%s does not depend on %s", what, on)
			return False
		elif depend_map[cur] == on:
			notice("%s depends on %s", what, on)
			return True
		else:
			cur = depend_map[cur]
	
	raise Exception, "Cyclic dependency, update '%s' depends on itself" % cur

def depend_cmp(what, on, depend_map):
	if depends(what, on, depend_map):
		return 1
	elif depends(on, what, depend_map):
		return -1
	else:
		return 0

class PreparationProgress:
	def __init__(self):
		self.template_install_request = None
		self.native_repos_updated = False
		self.templates = {}		# platform -> list of templates
		self.update_templates = {}	# VZContainer -> list of (template_id, name, version)
		self.remove_templates = {}	# template_id -> (name, version)
		self.updated_hosts = []
		self.windows_update = {}

class BuildInfo:
	def __init__(self, builds):
		self.depends_to_check = None
		self.present_updates = None
		self.upgrade_instructions = None
		self.builds = builds
		self.version_list = []
		self.progress = None
		self.conflicts = []

	def loadProgress(self, config):
		self.progress = uUtil.loadObj(config.cache, "progress.dat", PreparationProgress)

	def saveProgress(self, config):
		uUtil.saveObj(self.progress, config.cache, "progress.dat")

def getCumulativeUpdateInstructions(builds):
	filelist = {}
	for build in builds:
		for instruction in build.instructions:
			filelist[os.path.join(build.topdir, instruction)] = build
	progress.do("reading udl files")
	depend_map = {}
	name_filemap = {}
	fn_docmap = {}
	present_updates = sets.Set()
	required_updates = sets.Set()
	for fn in filelist:
		doc = dom.parse(fn)
		root = doc.documentElement
		name = root.getAttribute("name")
		depend = root.getAttribute("required")
		name_filemap[name] = fn
		fn_docmap[fn] = doc
		present_updates.update([name])

		if depend:
			depend_map[name] = depend
			required_updates.update([depend])
	
	ol = name_filemap.keys()[:]
	ol.sort(lambda x, y: depend_cmp(x, y, depend_map))
	
	if not ol:
		raise Exception, "No update instructions found"

	ordered_udlfiles = [ name_filemap[x] for x in ol]
	ordered_builds = [filelist[x] for x in ordered_udlfiles]
	notice("Updates will performed in following order: %s", '==>'.join(ordered_udlfiles))

	#ordered_docs = [ fn_docmap[x] for x in ordered_udlfiles ]

	binfo = BuildInfo(ordered_builds)
	binfo.depends_to_check = required_updates - present_updates
	binfo.present_updates = present_updates
	binfo.upgrade_instructions = Update()
	for doc in ordered_udlfiles:
		binfo.version_list.append(binfo.upgrade_instructions.process(fn_docmap[doc].documentElement, doc))

	progress.done()

	return binfo

_config = None

def configureUDLReader(conf):
	global _config
	_config = conf

__all__ = [ "Update", "Instruction", "SQLScript", "ExecScript", "PkgInstallation", "getCumulativeUpdateInstructions", "configureUDLReader", 'BuildInfo' ]

