import uSysDB

def _addEventParameter(con, event_type_id, name, description):
	cur = con.cursor()
	cur.execute(
	    "INSERT INTO em_event_params (event_type_id, name, descr) "
	    "VALUES(%s, %s, %s)", event_type_id, name, description)

def registerEvent(con, event_type_id, name, description, e_system, parameters):
	cur = con.cursor()
	cur.execute(
	    "INSERT INTO em_events(event_type_id, name, descr, e_system) "
	    "VALUES  (%s, %s, %s, %s) ", event_type_id, name, description, e_system)
	for param in parameters:
		_addEventParameter(con, event_type_id, param[0], param[1])

def unregisterEvent(con, event_type_id):
	cur = con.cursor()
	# cascade deletion from em_calls, em_call_params, etc.
	cur.execute(
	    "DELETE FROM em_event_handlers WHERE event_type_id = %s", event_type_id)
	# cascade deletion from em_event_params
	cur.execute(
	    "DELETE FROM em_events WHERE event_type_id = %s", event_type_id)

def _addActionParam(con, act_id, name, value):
	cur = con.cursor()
	cur.execute(
	    "INSERT INTO em_action_params (act_id, name, value) VALUES(%s, %s, %s)", act_id, name, value)

def _registerAction(con, act_type_id, name, description, friendly, parameters):
	cur = con.cursor()
	cur.execute(
	    "INSERT INTO em_actions (act_type_id, name, descr, friendly) "
	    "VALUES(%s, %s, %s, %s)", act_type_id, name, description, friendly)
	_act_id = uSysDB.get_last_inserted_value(con, 'em_actions')
	for param in parameters:
		_addActionParam(con, _act_id, param[0], param[1])
	return _act_id;

def _addMessageType(con, name, description):
	cur = con.cursor()
	cur.execute(
	    "INSERT INTO message_types(name, description) "
	    "VALUES (%s, %s)", name, description)
	return uSysDB.get_last_inserted_value(con, 'message_types')

def _addMessage(con, message):
	cur = con.cursor()
	cur.execute(
	    "INSERT INTO messages (message_key) VALUES (%s)", message)
	return uSysDB.get_last_inserted_value(con, 'messages')

def _addEventHandler(con, event_type_id, owner_id):
	cur = con.cursor()
	cur.execute(
	    "INSERT INTO em_event_handlers   (event_type_id, owner_id) "
	    "VALUES (%s, %s)", event_type_id, owner_id)
	return uSysDB.get_last_inserted_value(con, 'em_event_handlers')

def _addCallParam(con, call_id, name, value):
	cur = con.cursor()
	cur.execute(
	    "INSERT INTO em_call_params (call_id, name, value) VALUES (%s, %s, %s)", call_id, name, value)

def _addCall(con, hand_id, act_id, parameters):
	cur = con.cursor()
	cur.execute(
	    "INSERT INTO em_calls (hand_id, act_id) "
	    "VALUES (%s, %s)", hand_id, act_id)
	_call_id = uSysDB.get_last_inserted_value(con, 'em_calls')
	for param in parameters:
		_addCallParam(con, _call_id, param[0], param[1])
	return _call_id

def registerNotificationMessage(con, event_type_id, subj, body):
	cur = con.cursor()
	# get event name and description
	cur.execute("SELECT name, descr FROM em_events WHERE event_type_id = %s", event_type_id)
	row = cur.fetchone()
	if row is None:
		# exception
		raise "Event type with id: %d not found"%event_type_id

	event_name = row[0]
	event_descr = row[1]

	# new message registration
	msg_type_id = _addMessageType(con, event_name, event_descr)
	act_id = _registerAction(
	    con,
	    9000,	# it is IDL constant for 'Send notification action'
	    'Send ' + event_name + ' notification',
	    event_descr, 1,
	    [ ('context_vendor_id', '${context_vendor_id}'),
	      ('account_id_list', '${context_account_id},${context_vendor_id},${actor_account_id}'),
	      ('message_type', str(msg_type_id))] )
	subj_msg_id = _addMessage(con, subj)
	body_msg_id = _addMessage(con, body)
	cur.execute(
	    "UPDATE message_types SET subject_msg_id = %s, body_msg_id = %s, action_id = %s "
	    "WHERE message_type_id = %s", subj_msg_id, body_msg_id, act_id, msg_type_id)
	hand_id = _addEventHandler(con, event_type_id, 0)
	call_id = _addCall(con, hand_id, act_id,
	    [ ('account_id_list', '${context_account_id},${context_vendor_id},${actor_account_id}'),
	      ('context_vendor_id', '${context_vendor_id}'),
	      ('message_type', str(msg_type_id))])

def unregisterNotificationMessage(con, event_type_id):
	cur = con.cursor()
	cur.execute("SELECT name FROM em_events WHERE event_type_id = %s", event_type_id)
	row = cur.fetchone()
	if row is None:
		# no event registered, nothing to do
		return

	event_name = row[0]

	# drop message_type
	cur.execute(
	    "SELECT subject_msg_id, body_msg_id, action_id FROM message_types WHERE name = %s",
	    event_name)
	row = cur.fetchone()
	if row is None:
		# no message registered
		return

	subj_msg_id = row[0]
	body_msg_id = row[1]
	act_id = row[2]
	cur.execute("DELETE FROM em_actions WHERE act_id = %s", act_id);
	for msg_id in [subj_msg_id, body_msg_id]:
		cur.execute(
		    "DELETE FROM messages WHERE message_id = %s", msg_id)

	cur.execute("DELETE FROM message_types WHERE name = %s", event_name)

__all__ = [
	'registerEvent',
	'registerNotificationMessage',
	'unregisterNotificationMessage',
	'unregisterEvent' ]
