from threading import *

class Result:
	def __init__(self, success, result):
		self.value = result
		self.success = success
	
	def error(self):
		if self.success:
			return None
		else:
			return self.value

class CompletionSync:
	def __init__(self):
		self.threads = None
		self.result_map = {}
		self.lock = Lock()
		self.now_wait_for = None
		self.complete_threads = []
	
	def thread_complete(self, arg, result):
		try:
			self.lock.acquire()
			self.result_map[arg] = result
		finally:
			self.lock.release()
	
	def wait(self, threads):
		self.threads = threads
		try:
			self.lock.acquire()
			
			while len(self.threads):
				self.now_wait_for = self.threads[0]
				self.threads.remove(self.now_wait_for)
				self.lock.release()
				self.now_wait_for.join()
				self.lock.acquire()
		finally:
			self.lock.release()
			return self.result_map

class Worker(Thread):
	def __init__(self, fun, arg, args, sync):
		Thread.__init__(self)
		self.fun = fun
		self.arg = arg
		self.args = args
		self.sync = sync

	def run(self):
		try:
			result = Result(True, self.fun(self.arg, *self.args))
		except Exception, e:
			result = Result(False, e)
	
		self.sync.thread_complete(self.arg, result)

def doForEach(fun, argmap, parallel = False):
	result = {}
	if parallel:
		sync = CompletionSync()
		workers = [ Worker(fun, x, argmap[x], sync) for x in argmap]
		for worker in workers:
			worker.start()
		result = sync.wait(workers)

	else:	
		for arg in argmap:
			try :
				result[arg] = Result(True, fun(arg, *argmap[arg]))
			except Exception, e:
				result[arg] = Result(False, e)
	
	return result
	

__all__ = [ "doForEach", "Result" ]
