__rcs_id__ = """$Id: uHCL.py 727107 2012-01-12 10:30:13Z vvinogradov $"""

__pychecker__ = 'unusednames=__rcs_id__,dummy maxargs=16'

from xml.dom import minidom as dom
import uPEM, uSysDB, uLogging

def bool_str(val):
	if val:
		return "yes"
	else:
		return "no"

def setOptionalAttr(elem, name, value):
	if value is not None:
		elem.setAttribute(name, value)

class Request:

	def __init__(self, host_id = None, user=None, group = None, auto_export = True):
		self.__host_id = host_id
		self.auto_export = auto_export

		self.clear()
		self.set_creds(user, group)
	
	def clear(self):
		impl = dom.getDOMImplementation()
		dt = impl.createDocumentType("HCL", "HCL 1.0", "HCL")
		self.__document = impl.createDocument("HCL", "HCL", dt)
		self.__declare = self.__document.createElement("DECLARE")
		self.__document.documentElement.appendChild(self.__declare)
		self.__perform = self.__document.createElement("PERFORM")
		self.__document.documentElement.appendChild(self.__perform)

	def set_creds(self, user=None, group = None):
		if user is not None:
			self.set("default_user", user)
		if group is not None:
			self.set("default_group", group)


	def __setVarAttr(self, elem, attname, varname):
		if varname is None:
			return

		if self.auto_export:
			self.export(varname)

		elem.setAttribute(attname, varname)


	def mkfile(self, path, contents, owner="${default_user}", group="${default_group}", perm=None, overwrite = True):
		elem = self.__document.createElement("CREATEFILE")
		elem.setAttribute("path", path)
		elem.setAttribute("owner", owner)
		elem.setAttribute("group", group)
		setOptionalAttr(elem, "perm", perm)
		elem.setAttribute("overwrite", bool_str(overwrite))
		text = self.__document.createCDATASection(contents)
		elem.appendChild(text)
		self.__perform.appendChild(elem)


	def set(self, name, value):
		elem = self.__document.createElement("SET")
		elem.setAttribute("var", name)
		elem.setAttribute("value", value)
		self.__perform.appendChild(elem)

	def export(self, name, value = None, transient = False):
		elem = self.__document.createElement("VAR")
		elem.setAttribute("name", name)
		if value is not None:
			elem.setAttribute("value", value)
		elem.setAttribute("transient", bool_str(transient))

		self.__declare.appendChild(elem)
	
	def command(self, command, args = None, stdout = None, stderr = None, cwd='/', stdin = None, user="${default_user}", group="${default_user}", valid_exit_codes = None, retvar= None):
		if args is None:
			args = [ "${default_shell_name}", "${shell_cmd_switch}", command ]
			command = "${default_shell_name}"
		elem = self.__document.createElement("EXEC")
		elem.setAttribute("command", command)
		elem.setAttribute("workdir", cwd)
		setOptionalAttr(elem, "user", user)
		setOptionalAttr(elem, "group", group)
		self.__setVarAttr(elem, "outvar", stdout)
		self.__setVarAttr(elem, "errvar", stderr)
		self.__setVarAttr(elem, "retvar", retvar)
		if valid_exit_codes is not None:
			elem.setAttribute("valid_exit_codes", ','.join([str(x) for x in valid_exit_codes ]))
		for arg in args:
			arg_elem = self.__document.createElement("ARG")
			arg_elem.setAttribute("value", arg)
			elem.appendChild(arg_elem)
		if stdin is not None:
			in_elem = self.__document.createCDATASection(stdin)
			elem.appendChild(in_elem)

		self.__perform.appendChild(elem)
	
	def fetch(self, srcfile, urls, dstfile = None, dstvar = None):
		elem = self.__document.createElement("FETCH")
		elem.setAttribute("srcfile", srcfile)

		for url in urls:
			lelem = self.__document.createElement("LOCATION")
			lelem.setAttribute("value", url)
			elem.appendChild(lelem)

		if dstfile is None:
			elem.setAttribute("dstfile", "")
		else:
			elem.setAttribute("dstfile", dstfile)

		self.__setVarAttr(elem, "tmpfilevar", dstvar)

		self.__perform.appendChild(elem)


	def mkdir(self, directory):
		elem = self.__document.createElement("MKDIR")
		elem.setAttribute("path", directory)
		self.__perform.appendChild(elem)

	def rm(self, path):
		elem = self.__document.createElement("RM")
		elem.setAttribute("path", path)
		self.__perform.appendChild(elem)

	def move(self, src, dst):
		elem = self.__document.createElement("MOVE")
		elem.setAttribute("src", src)
		elem.setAttribute("dst", dst)
		elem.setAttribute("overwrite", "yes")
		self.__perform.appendChild(elem)

	def copy(self, src_dir, dst_dir):
		elem = self.__document.createElement("COPY")
		elem.setAttribute("src", src_dir)
		elem.setAttribute("dst", dst_dir)
		self.__perform.appendChild(elem)

	def chown(self, path, user="${default_user}", group="${default_group}", recursive=True):
		elem = self.__document.createElement("CHOWN")
		elem.setAttribute("path", path)
		elem.setAttribute("recursive", bool_str(recursive))
		elem.setAttribute("user", user)
		elem.setAttribute("group", group)
		self.__perform.appendChild(elem)

	def chmod(self, path, perm, recursive=False, action="set"):
		elem = self.__document.createElement("CHMOD")
		elem.setAttribute("path", path)
		elem.setAttribute("perm", perm)
		elem.setAttribute("action", action)
		elem.setAttribute("recursive", bool_str(recursive))
		self.__perform.appendChild(elem)

	def readfile(self, path, content, binary = False):
		if binary:
			cmd = "READB64"
		else:
			cmd = "READ"
		elem = self.__document.createElement(cmd)
		elem.setAttribute("path", path)
		self.__setVarAttr(elem, "variable", content)

		self.__perform.appendChild(elem)

	def extract(self, archive, rootpath):
		elem = self.__document.createElement("UNPACK")
		elem.setAttribute("filename", archive)
		elem.setAttribute("rootpath", rootpath)

		self.__perform.appendChild(elem)

	def transfer(self, src_host_id, pathsubst):
		if not sum([len(x) for x in pathsubst]):
			return

		ctri = self.__getCTRI(src_host_id) 

		for x in pathsubst:
			self.__transfer(ctri, x[0], x[1])

	def perform(self, host_id = None):
		host_id = host_id or self.__host_id
		if host_id is None:
			raise Exception("host_id not defined")

		con = uSysDB.connect()
		p, path = uPEM.getHostInfo(con, host_id)
		if p.os == 'Win32':
			shell = "cmd.exe"
			cmd_switch = "/C"
		else:
			shell = "/bin/sh"
			cmd_switch = "-c"

		self.export("default_shell_name", shell, True)
		self.export("shell_cmd_switch", cmd_switch, True)
		try:
			return uPEM.performHCLRequest(host_id, self.__document)
		except uPEM.ExecFailed, e:
			if e.err.find("OBJ_ADAPTER") >= 0 or e.err.find("OBJECT_NOT_EXIST") >= 0:
				uLogging.notice("Retrying call")
				return uPEM.performHCLRequest(host_id, self.__document)
			else:
				raise

	def __transfer(self, ctri, path_from, path_to):
		elem = self.__document.createElement("TRANSFER")
		elem.setAttribute("ctri", ctri)

		dir_el = self.__document.createElement("PATHSUBST")
		dir_el.setAttribute("what", path_from)
		dir_el.setAttribute("to", path_to)
		elem.appendChild(dir_el)

		self.__perform.appendChild(elem)

	def __getCTRI(self, host_id):
		h = uPEM.getHost(host_id)
		ior = self.__getPleskdIOR(h.pleskd_id)
		return 'CTRI: VE_ID:0 IOR:%s PLATFORM:%s' % (ior, h.platform.os)

	def __getPleskdIOR(self, pleskd_id):
		con = uSysDB.connect()
		cur = con.cursor()
		if uSysDB.hamn_schema():
			cur.execute("SELECT ior FROM service_references sr JOIN interfaces i ON (sr.interface_id = i.interface_id) JOIN sc_instances si ON (sr.sc_instance_id = si.sc_instance_id) WHERE si.sc_id = %s AND i.service_type = %s", pleskd_id, 'pleskd')
		else:
			cur.execute("SELECT ior FROM service_references sr JOIN interfaces i ON (sr.interface_id = i.interface_id) WHERE sr.sc_id = %s AND i.service_type = %s", pleskd_id, 'pleskd')
		return cur.fetchone()[0]
