__rcs_id__ = """$Id: uIPTranslation.py 564938 2010-04-01 05:40:44Z mkhon $"""

__pychecker__ = 'unusednames=__rcs_id__,dummy maxargs=16'
import uSysDB, re

ip_pattern = re.compile(r"(\d+).(\d+).(\d+).(\d+)")

def parse_ip(ip):
	pmatch = ip_pattern.match(ip)
	if not pmatch:
		raise Exception("%s: invalid ip address" % ip)

	return int(pmatch.group(1)), int(pmatch.group(2)), int(pmatch.group(3)), int(pmatch.group(4))
	
def var_ip(ip):
	digits = parse_ip(ip)
	return "%d.%d.%d.%d" % digits

def ip16(ip):
	return "%03d.%03d.%03d.%03d" % parse_ip(ip)

def ip_to_int(ip):
	digits = parse_ip(ip)
	rv = 0
	for d in digits:
		rv <<= 8
		rv += d
	return rv

def int_to_ip(i):
	d1 = i & 255
	i >>= 8
	d2 = i & 255
	i >>= 8
	d3 = i & 255
	i >>= 8
	d4 = i & 255
	return "%d.%d.%d.%d" % (d4, d3, d2, d1)


class IPTranslation:
	def __init__(self, con):
		self.network_rules = []
		self.existant_trs = {}
		self.con = con

		cur = self.con.cursor()

		cur.execute("SELECT fromip_frontnet, toip_frontnet, fromip_publicnet, toip_publicnet FROM iptr_rules");
		for row in cur.fetchall():
			front_range = (ip_to_int(row[0]), ip_to_int(row[1]))
			pub_range = (ip_to_int(row[2]), ip_to_int(row[3]))
			self.network_rules.append((front_range, pub_range))

	
	def add_tr(self, tr_type, ip_front, ip_pub, port):
		cur = self.con.cursor()
		cur.execute("INSERT INTO iptr_translations (iptr_type, ip_publicnet, ip_frontnet, ip16_publicnet, ip16_frontnet, description, iptr_used) VALUES(%s, %s, %s, %s, %s, '', 1)", tr_type, var_ip(ip_front), var_ip(ip_pub), ip16(ip_front), ip16(ip_pub))
		tr_id = uSysDB.get_last_inserted_value(self.con, "iptr_translations")
		cur.execute("INSERT INTO iptr_translations_ports(iptr_id, port_publicnet, port_frontnet) VALUES (%s, %s, %s)", tr_id, port, port)
		self.existant_trs[(ip_front, port)] = tr_id
		return tr_id


	def get_translation(self, ip, port):
		ip = var_ip(ip)
		cur = self.con.cursor()
		cur.execute("SELECT tr.iptr_id, tr.iptr_used FROM iptr_translations tr JOIN iptr_translations_ports trp ON (tr.iptr_id = trp.iptr_id) WHERE tr.ip_frontnet = %s AND trp.port_frontnet = %s AND tr.iptr_type = 'd'", ip, port)
		row = cur.fetchone()

		if row:
			tr_id = row[0]
			if not row[1]:
				cur.execute("UPDATE iptr_translations SET iptr_used = 1 WHERE iptr_id = %s", tr_id)
			self.existant_trs[(ip, port)] = tr_id
			return tr_id

		cur.execute("SELECT iptr_id, iptr_used FROM iptr_translations WHERE iptr_type = 'w' AND ip_frontnet = %s", ip)
		row = cur.fetchone()
		if row:
			tr_id = row[0]
			if not row[1]:
				cur.execute("UPDATE iptr_translations SET iptr_used = 1 WHERE iptr_id = %s", tr_id)
			self.existant_trs[(ip, port)] = tr_id
			cur.execute("SELECT 1 FROM iptr_translations_ports WHERE iptr_id = %s AND port_frontnet = %s", tr_id, port)
			if not cur.fetchone():
				cur.execute("INSERT INTO iptr_translations_ports(iptr_id, port_publicnet, port_frontnet) VALUES (%s, %s, %s)", tr_id, port, port)
			return tr_id

		# no translations, check rules:

		int_ip = ip_to_int(ip)
		to_ip = None
		for front_r, public_r in self.network_rules:
			b, e = front_r
			print int_to_ip(b), int_to_ip(e), int_to_ip(int_ip)
			if b <= int_ip <= e:
				to_b, to_e = public_r
				to_ip = int_to_ip(to_b + (int_ip - b))

		if to_ip is not None:
			return self.add_tr('w', to_ip, ip, port)
		else:
			return self.add_tr('d', ip, ip, port)

	def require_translation(self, ip, port, consumer):
		cur = self.con.cursor()
		tr_id = self.get_translation(ip, port)
		cur.execute("SELECT 1 FROM iptr_consumers WHERE iptr_id = %s AND port_frontnet = %s AND consumer = %s", tr_id, port, consumer)
		row = cur.fetchone()
		if not row:
			cur.execute("INSERT INTO iptr_consumers (iptr_id, port_frontnet, consumer) VALUES(%s, %s, %s)", tr_id, port, consumer)

