from uAction import *
from uDLReader import *
import os, sys,  re,  cStringIO, gc, tempfile, shutil
from uLogging import err, warn, notice
from uConfig import Config
from xml.dom import minidom as dom
import __builtin__

def tryCheckSQL(instructions):
	actions = [ x for x in instructions.prepreparation.actions if isinstance(x, SQLScript) ]
	actions += [ x for x in instructions.preparation.actions if isinstance(x, SQLScript) ]
	actions += [ x for x in instructions.pre.actions if isinstance(x, SQLScript) ]
	actions += [ x for x in instructions.actions if isinstance(x, SQLScript) ]
	actions += [ x for x in instructions.post.actions if isinstance(x, SQLScript) ]
	actions += [ x for x in instructions.cleanup.actions if isinstance(x, SQLScript) ]

	errors = False
	for action in actions:
		for stmt in action.parsed_code():
			kind, text = stmt
			if kind in ('CREATE', 'ALTER', 'DROP'):
				err("Action:%s, not portable statement '%s'", action.id, stmt)
				errors = True
			elif kind in ('BEGIN', 'COMMIT', 'ROLLBACK'):
                                warn('%s statements are ignored, <SQL> action always implicitely begins and commits transaction', kind)

	
	if errors and not (config and config.force):
		raise Exception('Non portable SQL in some actions')

pychecker_import = None
orig_import = None

def tryCheckExec(instructions):
	os.environ['PYCHECKER'] = '--limit 666 --maxlines 666 --maxlocals 666'
	try:
		global pychecker_import, orig_import
		if pychecker_import is None:
			orig_import = __builtin__.__import__
			import pychecker.checker as checker
			pychecker_import = __builtin__.__import__
		else:
			__builtin__.__import__ = pychecker_import
	except ImportError, e:
		err("Pychecker is not installed, not checking EXEC")
		return
	
	old_sys_path = sys.path
	tempdir = tempfile.mkdtemp()
	sys.path = [tempdir] + sys.path

	actions = [ x for x in instructions.prepreparation.actions if isinstance(x, ExecScript) ]
	actions += [ x for x in instructions.preparation.actions if isinstance(x, ExecScript) ]
	actions += [ x for x in instructions.pre.actions if isinstance(x, ExecScript) ]
	actions += [ x for x in instructions.actions if isinstance(x, ExecScript) ]
	actions += [ x for x in instructions.post.actions if isinstance(x, ExecScript) ]
	actions += [ x for x in instructions.cleanup.actions if isinstance(x, ExecScript) ]

	module_fn = os.path.join(tempdir, 'update_actions.py')
	notice("Using temporary actions file %s", module_fn)
	module = open(module_fn, 'w+')

	line_mapping = []
	ln = 1
	for action in actions:
		fcode = action.makeFunctionCode('fn_'+str(ln))
		numlines = len(fcode.splitlines())
		line_mapping.append(((ln, ln + numlines), action.id))
		module.write(fcode)
		ln += numlines

	module.close()
	# Here it is!
	out_str = cStringIO.StringIO()
	sys.stdout = out_str
	import update_actions as dummy
        __builtin__.__import__ = orig_import
	sys.path = old_sys_path
	sys.stdout = sys.__stdout__
	shutil.rmtree(tempdir)
# /tmp/update_actions.py:49: (dir) shadows builtin
	err_pattern = re.compile(r'\w?:?[^:]+:(\d+):(.+)$')
	errors = out_str.getvalue().splitlines()

	for errline in errors:
		match = err_pattern.match(errline)
		if not match:
			warn("Cannot parse %s", errline)
		else:
			line = int(match.group(1))
			errmsg = match.group(2)
			found = False
			for mi in line_mapping:
				lr, aid = mi
				lb, le = lr
				if line >= lb and line < le:
					err("Action %s:%d:%s", aid, line - lb, errmsg)
					found = True
			if not found:
				err("Internal error - cannot find line %d", line)
	
	if errors and instructions.version > '2.6.3' and not (config and config.force):
		raise Exception("There are errors in upgrade procedure")

config = None

def init(conf):
	global config
	config = conf

if __name__ == "__main__":
	c = Config([
		('log:syslog_ident', str, None, "save_config"),
		('log:syslog_facility', str, 's', None),
		('log:file', str, 'l', None),
		('warning:duplicates', bool, None, True),
		('verbose', bool, 'v', False),
		('force', bool, 'f', False)
	], sys.argv, None, '/root/updater.cfg', (1, None), """Usage: %s udl2file [udl2file..]
"""
);
	init(c)
	udl_files=config.args

	if sys.platform != 'win32':
		invalid_udls = [ x for x in udl_files if os.system("xmllint -o /dev/null --dtdvalid %s %s" % (os.path.join(os.path.dirname(os.path.dirname(sys.argv[0])), 'UDL2.dtd'), x)) ]

		if invalid_udls:
			raise Exception("Following udl files are invalid: %s" % ", ".join(invalid_udls));

	update = Update()
	for udl in udl_files:
		doc = dom.parse(udl)
		update.process(doc.documentElement, udl)

	tryCheckSQL(update)
	tryCheckExec(update)
