import sys, codecs, time, logging

try:
	import syslog
	have_syslog = True
	prio_syslog = { "ERROR" : syslog.LOG_ERR, "WARNING": syslog.LOG_WARNING, "INFO" : syslog.LOG_INFO, "DEBUG" : syslog.LOG_DEBUG }
except ImportError:
	have_syslog = False

def print_log(prio, message, *msg_args):
	if len(msg_args) == 0:
		msg = message
	else:
		msg = (message % msg_args)
	if prio:
		to_write = "%s: %s\n" % (prio, msg)
	else:
		to_write = msg + '\n'
	if do_time:
		to_write = time.asctime() + ': ' + to_write
	to_write_encoded = to_write.encode('utf8')
	if logfile:
		logfile.write(to_write_encoded)
		logfile.flush()

	if do_syslog:
		syslog.syslog(prio_syslog[prio], to_write)
	if log_stderr:
		if isinstance(sys.stderr, codecs.Codec):
			sys.stderr.write(to_write)
		else:
			sys.stderr.write(to_write_encoded)

	if ext_log_dump:
		ext_log_dump(to_write)

def log_func(prio):
	def rv(message, *args):
		try: 
			return print_log(prio, message, *args)
		except IOError, (errno, strerror): 
			if errno != 11: 
				raise IOError, (errno, strerror)
	return rv

warn = log_func("WARNING")
err = log_func("ERROR")
debug = log_func("DEBUG")
notice = log_func("NOTICE")
log = log_func(None)

logfile = None
do_syslog = None
log_stderr = True
do_time = None

ext_log_dump = None

# handler for python logging library
class Handler(logging.Handler):
	def __init__(self):
		logging.Handler.__init__(self)

	def __get_logging_func(self, levelname):
		# default python logging doesn't define any more log levels besides standard DEBUG, INFO ..
		# but there is for example yum, which defines some more levels like INFO_1, INFO_2
		# so here we try to treat all the INFO_X levels as INFO
		if levelname.startswith("DEBUG"):
			return debug
		elif levelname.startswith("INFO"):
			return notice
		elif levelname.startswith("WARNING"):
			return warn
		elif levelname.startswith("ERROR"):
			return err
		elif levelname.startswith("CRITICAL"):
			return err

	def emit(self, record):
		self.__get_logging_func(record.levelname)(record.getMessage())
		
def init(config):
	if have_syslog and config.log.syslog_facility:
		if hasattr(syslog, config.log.syslog_facility):
			sf = getattr(syslog, config.log.syslog_facility)
			print sf, config.log.syslog_ident
			syslog.openlog(config.log.syslog_ident, syslog.LOG_PID, sf)
		global do_syslog
		do_syslog = True
	if config.log.file:
		global logfile
		logfile = file(config.log.file, "a")
	
	if hasattr(config, "verbose") and not config.verbose:
		def empty_fun(*args):
			pass
		global debug
		debug = empty_fun

__all__ = [ "warn" , "err", "debug", "notice", "log" ]
