__rcs_id__ = """$Id: uMultipleManagementNodes.py 643120 2011-03-16 02:58:26Z akalinin $"""

import uLogging
import openapi
from uUtil import *
import uSysDB

import os.path
import pickle
import time

cache_directory = None

class Storable:
	def load(cls):
		global cache_directory
		return loadObj(cache_directory, cls.filename(), lambda: None)

	load = classmethod(load)

	def store(self):
		global cache_directory
		saveObj(self, cache_directory, self.filename())

class MNInfo(Storable):
	def __init__(self, api):
		self.mns = api.pem.system.getManagementNodes()
		self.scs = api.pem.system.getSingletonServiceControllers()

	filename = classmethod(lambda cls: "multimn.cfg")

	def nodes(self):
		return self.__wrap(self.mns)

	def singletons(self):
		return self.__wrap(self.scs)

	def current(self):
		return [ x.host_id for x in self.nodes() if x.current ][0]

	def pretty_print(self):
		print "Management Nodes:"
		for mn in self.nodes():
			print "\thost_id:\t%d" % mn.host_id
			print "\toperational:\t%s" % mn.operational
			print "\tcurrent:\t%s" % mn.current
			print

		print "Singletons:"
		for si in self.singletons():
			print "\tsc_id:\t\t%d" % si.sc_id
			print "\thost_id:\t%d" % si.host_id
			print

	def __wrap(self, arr):
		return [ DictToStruct(x) for x in arr ]

class SuspendComplete(Storable):
	def __init__(self):
		self.complete = True

	filename = classmethod(lambda cls: "suspend.info")

def storeSuspendComplete():
	SuspendComplete().store()

def alreadySuspended():
	s = SuspendComplete.load() 
	return s is not None and s.complete

class Operation:
	def __init__(self, name, api):
		self.api = api
		self.name = name
		self.request_id = "updater.multimn.%s.%s" % (name, time.ctime().replace(' ', '-'))

	def perform(self, steps):
		self.api.begin(request_id = self.request_id)

		try:
			[ op() for op in steps ]
		except Exception, e:
			try:
				self.api.rollback()
			except:
				pass
			raise e

		self.api.commit()

	def wait(self):
		openapi.waitRequestComplete(self.request_id, self.name, True)

def suspendSpareManagementNodes(api):
	if alreadySuspended():
		uLogging.log("Spare management nodes already suspended")
		return

	info = MNInfo(api)
	info.store()
	current = info.current()

	for sc in info.singletons():
		if sc.host_id != current:
			api.pem.system.moveSingletonServiceController(sc_id = sc.sc_id, dest_host_id = current)

	for mn in info.nodes():
		if not mn.current:
			op = Operation('suspend-mn-%d' % mn.host_id, api)
			uLogging.log("Suspend management node #%d", mn.host_id)
			op.perform( [ lambda : api.pem.system.suspendManagementNode(host_id = mn.host_id) ] )
			op.wait()

	storeSuspendComplete()

def resyncSpareManagementNodes(api):
	if not alreadySuspended():
		uLogging.log("Nothing to do")
		return

	cached_info = MNInfo.load()
	curr_info = MNInfo(api)

	suspended_nodes = [ x.host_id for x in cached_info.nodes() if not x.current and x.operational ]
	uLogging.log("Resync suspended mangement nodes (in background)")
	api.pem.system.resyncManagementNodes(resume_hosts = suspended_nodes)

def systemHasMultipleManagementNodes():
	if not uSysDB.hamn_schema():
		return False

	con = uSysDB.connect()
	cur = con.cursor()
	cur.execute("SELECT count(*) FROM management_nodes WHERE (status IS NULL OR status != 'd')")
	return int(cur.fetchone()[0]) > 1

def init(config):
	global cache_directory
	cache_directory = config.cache

__all__ = [ "init", "systemHasRunningSpareManagementNodes", "MNInfo", "suspendSpareManagementNodes", "resumeSpareManagementNodes" ]
