#!/usr/bin/python

#
# $Id: uNativeRepo.py 554983 2010-02-16 12:55:01Z mkhon $
#

import sys
import getopt
import time
import openapi
import xmlrpclib as xmlrpc

_log_device = None

def log(s):
	if _log_device != None:
		_log_device.write(s)

def print_usage_exit(exit_code):
	print \
	"Usage:\n\t", sys.argv[0], \
	"[--openapi=URL] --create --os=OS --osver=OSVER --arch=ARCH --host_id=HOST-ID\n", \
	"\t", sys.argv[0], "[--openapi=URL] --index --repo_id=REPO-ID\n", \
	"\t", sys.argv[0], "[--openapi=URL] --info --repo_id=REPO-ID\n"

	sys.exit(exit_code)

def parse_args():
	try:
		opts, args = getopt.getopt(sys.argv[1:], "h", \
			["help", "create", "index", "info", \
			"openapi=", \
			"os=", "osver=", "arch=", \
			"host_id=", "repo_id="])

		global _openapi_url
		global _action
		params = {}
		repo_type = {}

		for o, v in opts:
			if o == "--openapi":
				openapi.init(v)

			if o in ("--create", "--index", "--info"):
				_action = o.lstrip('-')

			if o in ("-h", "--help"):
				print_usage_exit(0)

			if o == "--os":
				repo_type['opsys'] = v

			if o == "--osver":
				repo_type['osrel'] = v

			if o == "--arch":
				repo_type['arch'] = v

			if o == "--host_id":
				params['host_id'] = int(v)

			if o == "--repo_id":
				params['repo_id'] = int(v)

		if len(repo_type):
			repo_type['pkg_manager'] = 0 # yum
			params['repo_type'] = repo_type

		return params

	except getopt.error, e:
		print e
		print_usage_exit(2)

class CouldNotCreateRepo (Exception):
	def __init__(self, sec):
		Exception.__init__(self, "Native Repository did not created withing %d seconds" % sec)

class NativeRepoManager (openapi.OpenAPI):
	def create_repo(self, params, wait_until_created = True, time_wait = 60):
		repo = self.checkedCall(self.server.pem.packaging.native_repository.createRepository, params)

		param = { 'repo_id' : int(repo['result']['repo_id']) }
		status = {}

		if not wait_until_created:
			return repo

		# Wait for repository to be created
		delay_sec = 2
		max_cnt = time_wait / delay_sec

		for i in range(max_cnt):
			status = self.checkedCall(self.server.pem.packaging.native_repository.getRepository, param)
		
			if status['result']['is_created'] == 1:
				break;

			time.sleep(delay_sec);

		if status['result']['is_created'] != 1:
			raise CouldNotCreateRepo(delay_sec * max_cnt)

		return status

	def index_repo(self, params):
		return self.checkedCall(self.server.pem.packaging.native_repository.reindex, params)

	def get_repo_info(self, params):
		return self.checkedCall(self.server.pem.packaging.native_repository.getRepository, params)


def print_repo(repo):
	print('REPO_ID="%d" REPO_URL="%s"' % (repo['repo_id'], repo['url']))

def create_repo_print(mgr, params):
	params['is_local'] = xmlrpc.Boolean(0)
	log('creating native repository. params: ' + str(params) + '\n')
	repo = mgr.create_repo(params)
	print_repo(repo['result'])

def print_repo_info(mgr, params):
	log('getting repository access info, params: ' + str(params) + '\n')
	repo = mgr.get_repo_info(params)
	print_repo(repo['result'])

def index_repo(mgr, params):
	log('reindexing native repository, params: ' + str(params) + '\n')
	mgr.index_repo(params)


def main():
	params = parse_args()
	mgr = NativeRepoManager()

	global _log_device
	_log_device = sys.stderr

	try:
		actions = {
			"create" : create_repo_print,
			"index"  : index_repo,
			"info"   : print_repo_info
		}

		actions[_action](mgr, params)

	except Exception, e:
		print >> sys.stderr, "%s %s" % (e.__class__.__name__, e.args)
		sys.exit(1)

if __name__ == "__main__":
	main()
