import sys, os, time

from uAction import *
from uLogging import *
from sets import Set
from uBuild import Platform
import shutil

import xml.dom.minidom as dom
import PEMVersion

import tempfile
import re
import uSysDB

try:
        import subprocess as sp
except ImportError:
        # for temporary backward compatibility with centos 4 and python 2.3
        import pasubprocess as sp

from uUtil import *

def getAllHosts():
	con = uSysDB.connect()
	cur = con.cursor()
	cur.execute("SELECT h.host_id, h.primary_name, h.htype, p.opsys, p.osrel, p.arch, default_rootpath, h.pleskd_id FROM hosts h JOIN platforms p ON (p.platform_id = h.platform_id) WHERE h.deleting != 1")

	return [ PEMHost(row[0], row[1], row[2], Platform(row[3], row[4], row[5]), row[6], row[7]) for row in cur.fetchall() ]

def getHost(host_id):
	con = uSysDB.connect()
	cur = con.cursor()
	cur.execute("SELECT h.host_id, h.primary_name, h.htype, p.opsys, p.osrel, p.arch, default_rootpath, h.pleskd_id FROM hosts h JOIN platforms p ON (p.platform_id = h.platform_id) WHERE host_id = %s", host_id)

	row = cur.fetchone()
	return PEMHost(row[0], row[1], row[2], Platform(row[3], row[4], row[5]), row[6], row[7])
	

def preStopMN():
	con = uSysDB.connect()
	cur = con.cursor()

	if uSysDB.hamn_schema():
		cur.execute("UPDATE sc_instances SET pid = -1 WHERE sc_id IN (SELECT sc_id FROM service_classes WHERE name NOT IN ('pleskd', 'vzpemagent'))")
	else:
		cur.execute("UPDATE service_classes SET pid = -1 WHERE name NOT IN ('pleskd', 'vzpemagent')")

	con.commit()
	con.close()

def startMN_unix():
	readCmdExt(['killall', '-9', 'pleskd', 'SoLoader', 'pem'])
	time.sleep(0.5)
	execCommand('service pem start')

def stopMN_unix():
	preStopMN()
	readCmdExt(['killall', '-9', 'pleskd', 'SoLoader', 'pem'])

def stopMN_win():
	preStopMN()
	debug("%s %s %s" % readCmdExt(["net", "stop", "pem"]))

def startMN_win():
	stopMN_win()
	out, err, status = readCmdExt(['net', 'start', 'pem'])
	if status:
		raise Exception("failed to start pem %s %s %s" % (out, err, status))
 
if sys.platform == 'win32':
	stopMN = stopMN_win
	startMN = startMN_win
else:
	stopMN = stopMN_unix
	startMN = startMN_unix

def is_started(name):
	con = uSysDB.connect()
	cur = con.cursor()

	if uSysDB.hamn_schema():
		cur.execute("SELECT 1 FROM sc_instances si JOIN service_classes sc ON (si.sc_id = sc.sc_id) WHERE sc.name=%s AND si.pid > 0", name)
	else:
		cur.execute("SELECT 1 FROM service_classes WHERE name=%s AND pid > 0", name)

	return cur.fetchone()

def wait_for_sc(name, connection = None):
	if connection is None:
		con = uSysDB.connect()
	else:
		con = connection
	started= False
	progress.do("waiting for %s to start", name)
	cur= con.cursor()
	while not started:
		time.sleep(3)
		started = is_started(name)
		if not started:
			sys.stderr.write('.')

	cur.close()
	if connection is None:
		con.close()
	progress.done()

def wait_for_scs(sclist, connection = None, monitor = None):
	if not connection:
		con = uSysDB.connect()
		own = True
	else:
		con = connection
		own = False
	cur = con.cursor()
	not_started = Set(sclist)

	progress.do("waiting for %s to start", ', '.join(not_started))
	while not_started:
		if uSysDB.hamn_schema():
			cur.execute("SELECT DISTINCT name FROM service_classes sc JOIN sc_instances si ON (sc.sc_id = si.sc_id) WHERE si.pid > 0")
		else:
			cur.execute("SELECT name FROM service_classes WHERE pid > 0")

		just_started = not_started.intersection(Set([x[0] for x in cur.fetchall()]))
		not_started -= just_started
		if monitor:
			monitor.started(list(just_started))
		else:
			sys.stderr.write(' '.join(just_started))
		if not_started:
			time.sleep(3)
			if not monitor:
				sys.stderr.write('.')
	cur.close()
	if own:
		con.close()
	
	progress.done()



def getSCProperty(con, scname, propname):
	cur = con.cursor()
	if uSysDB.hamn_schema():
		cur.execute("SELECT value FROM v_props v JOIN sc_instances si ON (v.component_id = si.component_id) JOIN service_classes sc ON (si.sc_id = sc.sc_id) WHERE sc.name = %s and v.name=%s", (scname, propname))
	else:
		cur.execute("SELECT value FROM v_props v JOIN service_classes sc ON (v.component_id = sc.component_id) where sc.name = %s and v.name=%s", (scname, propname))
	pval = cur.fetchone()
	if not pval:
		return None
	else:
		return pval[0]


def _setComponentProperty(component_id, pkg_id, propname, propvalue, description, cname, con):
	cur = con.cursor()
	cur.execute("SELECT prop_id FROM properties WHERE pkg_id = %s AND name = %s", (pkg_id, propname))

	row = cur.fetchone()

	if row:
		prop_id = row[0]
	else:
		if propname == 'db_passwd' or propname == 'dsn_passwd' or propname == 'windows.admin.user.password':
			protected='y'
		else:
			protected='n'

		cur.execute("INSERT INTO properties (pkg_id, name, default_value, description, protected) VALUES(%s, %s, %s, %s, %s)", (pkg_id, propname, propvalue, description, protected))
		prop_id = uSysDB.get_last_inserted_value(con, "properties")
	
	cur.execute("SELECT value_id FROM component_properties WHERE component_id = %s AND prop_id = %s", (component_id, prop_id))

	row = cur.fetchone()
	if row:
		cur.execute("UPDATE property_values SET value = %s WHERE value_id = %s", (propvalue, row[0]))
		notice("Updating already existent value %d for %s property %s", row[0], cname, propname)
	else:
		notice("Adding new value for %s property %s", cname, propname)
		cur.execute("INSERT INTO property_values(component_id, value) VALUES(%s, %s)", (component_id, propvalue))
		value_id = uSysDB.get_last_inserted_value(con, "property_values")
		cur.execute("INSERT INTO component_properties(component_id, value_id, prop_id ) VALUES(%s, %s, %s)", (component_id, value_id, prop_id))


def setComponentProperty(component_id, propname, propvalue, description = '', connection = None):
	if not connection:
		con = uSysDB.connect()
		own = True
	else:
		con = connection
		own = False

	cur = con.cursor()
	cur.execute("SELECT c.pkg_id, p.name, p.ctype, c.host_id FROM components c JOIN packages p ON (p.pkg_id = c.pkg_id) WHERE c.component_id = %s", component_id)

	row = cur.fetchone()
	if not row:
		raise Exception("%s: there is no such component installed", component_id)
	
	pkg_id = row[0]
	name = "[%s-%s on %d]" % (row[2], row[1], row[3])
	_setComponentProperty(component_id, pkg_id, propname, propvalue, description, name, con)

	if own:
		con.commit()


def setSCProperty(con, scname, propname, propvalue, description=''):
	cur = con.cursor()

	if uSysDB.hamn_schema():
		cur.execute("SELECT c.pkg_id, si.component_id FROM sc_instances si JOIN service_classes sc ON (sc.sc_id = si.sc_id) JOIN components c ON (c.component_id = si.component_id) WHERE sc.name=%s", scname)
	else:
		cur.execute("SELECT c.pkg_id, c.component_id FROM service_classes sc JOIN components c ON (c.component_id = sc.component_id) WHERE sc.name=%s", scname)

	for row in cur.fetchall():
		pkg_id = row[0]
		component_id = row[1]
		_setComponentProperty(component_id, pkg_id, propname, propvalue, description, scname, con)
	else:
		notice('sc %s is not installed, cannot set property for it', scname)

# 1. Should be used in PRE section only (in other case updater will not update SC binary and location)
# 2. If SC where pointed by IOR from tasks, specific upgrade shall be implemented
# as IOR contains SC name
# 3. IOR in service references dropped as it is changed (it contains SC name)
# 4. is not suitable for pleskd rename
def renameSC(con, old_scname, new_scname):
	cur = con.cursor()
	# drop IOR's they contain SC name
	if uSysDB.hamn_schema():
		cur.execute(
		    "DELETE FROM service_references WHERE sc_instance_id IN "
		    "(SELECT si.sc_instance_id FROM sc_instances si JOIN service_classes sc ON (si.sc_id = sc.sc_id) WHERE sc.name = %s)", old_scname)
	else:
		cur.execute(
		    "DELETE FROM service_references WHERE sc_id = (SELECT sc_id FROM service_classes WHERE name = %s)", old_scname)
	# we should rename old SC packages to be upgradable
	cur.execute(
	    "UPDATE packages SET name = %s WHERE name = %s AND ctype = 'sc'", new_scname, old_scname)
	# we should rename installed SC itself if any
	cur.execute(
	    "UPDATE service_classes SET name = %s WHERE name = %s", new_scname, old_scname)
	# we should update dependencies from renamed SC (pointed by name)
	cur.execute(
	    "UPDATE dep_packages SET name = %s WHERE name = %s AND ctype = 'sc'", new_scname, old_scname)

def setSCSingleton(con, scname):
	cur = con.cursor()
	cur.execute("SELECT 1 FROM sc_singletons ss JOIN service_classes sc ON (sc.sc_id = ss.sc_id) WHERE sc.name = %s", scname)
	row = cur.fetchall()

	if len(row):
		return

	cur.execute("SELECT host_id FROM management_nodes WHERE operational = %s ORDER BY host_id", "y")
	row = cur.fetchone()
	host_id = row[0]

	cur.execute("INSERT INTO sc_singletons (sc_id, sc_instance_id) SELECT sc.sc_id, si.sc_instance_id FROM service_classes sc JOIN sc_instances si ON (sc.sc_id = si.sc_id) JOIN components c ON (c.component_id = si.component_id) WHERE sc.name = %s AND c.host_id = %s", scname, host_id)

def getHostInfo(con, host_id):
	cur = con.cursor()
	cur.execute("SELECT p.opsys, p.osrel, p.arch, h.default_rootpath, p.platform_id FROM platforms p JOIN hosts h ON (h.platform_id = p.platform_id) WHERE h.host_id = %s", host_id)
	row = cur.fetchone()

	if not row:
		raise Exception("Database inconsistency - there is host with id %s!" % host_id)
	
	platform = Platform(row[0], row[1], row[2])
	platform.platform_id = row[4]
	rootpath = row[3]
	cur.close()
	return platform, rootpath

	
def default_sysdb_con(fun):
	argnames = fun.func_code.co_varnames[:fun.func_code.co_argcount]
	con_pos = None
	for ind in xrange(0, len(argnames)):
		if argnames[ind] == 'con':
			con_pos = ind
	def opt_con_fun(*args, **kwds):
		if len(args) <= con_pos and (not kwds.has_key('con') or kwds['con'] is None):
			own = True

			connection = uSysDB.connect()
			kwds['con'] = connection
		elif len(args) > con_pos and args[con_pos] is None:
			own = True
			connection = uSysDB.connect()
			args[con_pos] = connection
		else:
			own = False
		try:
			rv = fun(*args, **kwds)
			if own:
				connection.commit()
		finally:
			if own:
				connection.close()

		return rv
	
	return opt_con_fun

def default_ro_sysdb_con(fun):
	argnames = fun.func_code.co_varnames[:fun.func_code.co_argcount]
	con_pos = None
	for ind in xrange(0, len(argnames)):
		if argnames[ind] == 'con':
			con_pos = ind
	def opt_con_fun(*args, **kwds):
		if len(args) <= con_pos and (not kwds.has_key('con') or kwds['con'] is None):

			connection = uSysDB.connect()
			kwds['con'] = connection
		elif len(args) > con_pos and args[con_pos] is None:
			connection = uSysDB.connect()
			args[con_pos] = connection
		return fun(*args, **kwds)
	
	return opt_con_fun

getMNInfo = default_ro_sysdb_con(lambda con: getHostInfo(con, 1))

_installation_id = None

def installation_id(con = None):
	global _installation_id
	if _installation_id:
		return _installation_id
	

	if con is None:
		con = uSysDB.connect()
	cur = con.cursor()
	cur.execute("SELECT identity FROM installation_identity")
	row = cur.fetchone()

	if not row:
		_installation_id = raw_input("Please enter name for this installation, i.e. company name or brand:")
		_installation_id = _installation_id.strip()
		cur.execute("INSERT INTO installation_identity(identity) VALUES (%s)", _installation_id);
		con.commit();
	else:
		_installation_id = row[0]
	
	return _installation_id

	
def getPlatform(platform_id, con):
	cur = con.cursor()
	cur.execute("SELECT opsys, osrel, arch FROM platforms WHERE platform_id = %s", platform_id)
	row = cur.fetchone()
	if not row:
		return None
	p = Platform(row[0], row[1], row[2])
	p.platform_id = platform_id
	return p

def getPlatformLine(con, platform):
	cur = con.cursor()

	platform_id = platform.platform_id
	rv = [platform]
	while platform_id != 1:
		cur.execute("SELECT parent_id FROM platforms WHERE platform_id = %s", platform_id)
		row = cur.fetchone()
		if row is not None:
			platform_id = row[0]
		else:
			break
		p = getPlatform(platform_id, con)
		if p is not None:
			rv.append(p)
		else:
			break

	cur.close()
	return rv

_MN_info = None
__HCLHeader = """<?xml version="1.0" encoding="UTF-8" standalone="no" ?>
<!DOCTYPE HCL PUBLIC "HCL" "HCL 1.0">"""

prop_pattern=re.compile(r'([^=]+)=(.+)$')

class ExecFailed(Exception):
	def __init__(self, command, status, err):
		Exception.__init__(self, "%s exited with non-zero status %s saying %s" % (command, status, err))
		self.command = command
		self.status = status
		self.err = err

def readPropertyFile(name):
	global _MN_info
	if _MN_info is None:
		_MN_info = getMNInfo()
	platform, root = _MN_info

	rv = {}
	f = open(os.path.join(root, 'etc', name))
	for line in f:
		pmatch = prop_pattern.match(line)
		if pmatch:
			rv[pmatch.group(1)] = pmatch.group(2)
	return rv

def performHCLRequest(host_id, hcl):
	global _MN_info
	if _MN_info is None:
		_MN_info = getMNInfo()
	
	con = uSysDB.connect()
	cur = con.cursor()
	cur.execute("SELECT pleskd_id FROM hosts WHERE host_id = %s", host_id)
	row = cur.fetchone()
	if row:
		pleskd_id = row[0]
	else:
		pleskd_id = None
	cur.close()
	fd, filename  = tempfile.mkstemp()
	f = os.fdopen(fd, "w")
	if isinstance(hcl, dom.Document):
		# stupid toxml method does not include <DOCTYPE> to output.
		# do do it manually, and write only documentElement
		f.write(__HCLHeader)
		# do no pretty printing, because it can damage HCL request with whitespaces and newlines
		# (e.g. content of files to be created)
		hcl.documentElement.writexml(writer=f)
	else:
		f.write(hcl)
	f.close()

	platform, root = _MN_info
	if pleskd_id :
		command = [ os.path.join(root, 'bin', 'pleskd_ctl'), '-f', os.path.join(root, 'etc', 'pleskd.props'), '-s', str(pleskd_id), 'processHCL', filename, str(host_id)]
	else:
		command = [ os.path.join(root, 'bin', 'pleskd_ctl'), '-f', os.path.join(root, 'etc', 'pleskd.props'), '-H', str(host_id), 'processHCL', filename, str(host_id)]
	cmd = sp.Popen(command, stdout=sp.PIPE, stderr=sp.PIPE)
	out, err = cmd.communicate()
	status = cmd.returncode
	if status > 0:
		raise ExecFailed(command, status, err)
	elif status < 0:
		raise Exception, "%s is terminated by signal %s" % (command, -status)
	prop_values = out.split('\0')

	rv = {}
	for ind in xrange(0, len(prop_values) / 2):
		rv[prop_values[ind * 2]] = prop_values[ind * 2 + 1]

	# File is not removed if execCommand fail, and it is ok: this file could be needed to understand what went wrong.
	os.unlink(filename)

	return rv


def _execCtl(ctlname, fun, *args):
	global _MN_info
	if _MN_info is None:
		_MN_info = getMNInfo()
	platform, root = _MN_info
	
	if len(args) == 1 and type(args[0]) in (tuple, list):
		parameters = (args)[0]
	else:
		parameters = args
	
	command = [ os.path.join(root, 'bin', ctlname), '-f', os.path.join(root, 'etc', 'pleskd.props') ] + list(parameters)
	return fun(command)
	

def execCtl(ctlname, *args):
	return _execCtl(ctlname, execCommand, *args)

def readCtl(ctlname, *args):
	return _execCtl(ctlname, readCmd, *args)
	

linux_stopUI_HCL_old="""<?xml version="1.0" encoding="UTF-8" standalone="no" ?>
<!DOCTYPE HCL PUBLIC "HCL" "HCL 1.0">
<HCL>
<PERFORM>
<EXEC command="/bin/sh" user="root" group="root" >
<ARG value="/bin/sh" />
<ARG value="-c" />
<ARG value="export JAVA_HOME=%(java_home)s;/usr/local/pem/bin/.pem/apache/state_ctl stop; %(tomcat_home)s/bin/shutdown.sh;"/>
</EXEC>
</PERFORM>
</HCL>
"""

linux_startUI_HCL_old="""<?xml version="1.0" encoding="UTF-8" standalone="no" ?>
<!DOCTYPE HCL PUBLIC "HCL" "HCL 1.0">
<HCL>
<PERFORM>
<EXEC command="/bin/sh" user="root" group="root" >
<ARG value="/bin/sh" />
<ARG value="-c" />
<ARG value="export JAVA_HOME=%(java_home)s;/usr/local/pem/bin/.pem/apache/state_ctl start; %(tomcat_home)s/bin/startup.sh;"/>
</EXEC>
</PERFORM>
</HCL>
"""

linux_stopUI_HCL="""<?xml version="1.0" encoding="UTF-8" standalone="no" ?>
<!DOCTYPE HCL PUBLIC "HCL" "HCL 1.0">
<HCL>
<PERFORM>
<EXEC command="/bin/sh" user="root" group="root" >
<ARG value="/bin/sh" />
<ARG value="-c" />
<ARG value="killall -9 java || true" />
</EXEC>
</PERFORM>
</HCL>
"""

linux_startUI_HCL="""<?xml version="1.0" encoding="UTF-8" standalone="no" ?>
<!DOCTYPE HCL PUBLIC "HCL" "HCL 1.0">
<HCL>
<PERFORM>
<EXEC command="/bin/sh" user="root" group="root" >
<ARG value="/bin/sh" />
<ARG value="-c" />
<ARG value="service pemui start" />
</EXEC>
</PERFORM>
</HCL>
"""

win_stopUI_HCL="""<?xml version="1.0" encoding="UTF-8" standalone="no" ?>
<!DOCTYPE HCL PUBLIC "HCL" "HCL 1.0">
<HCL>
<PERFORM>
<EXEC command="cmd.exe" >
<ARG value="cmd.exe" />
<ARG value="/C" />
<ARG value="net stop pemui"/>
</EXEC>
</PERFORM>
</HCL>
"""

win_startUI_HCL="""<?xml version="1.0" encoding="UTF-8" standalone="no" ?>
<!DOCTYPE HCL PUBLIC "HCL" "HCL 1.0">
<HCL>
<PERFORM>
<EXEC command="cmd.exe" >
<ARG value="cmd.exe" />
<ARG value="/C" />
<ARG value="net start pemui"/>
</EXEC>
</PERFORM>
</HCL>
"""

def getUIStartupInfo(host_id, con):
	java_home = '/usr/java/jdk1'
	tomcat_home = '/usr/local/jakarta-tomcat-4.1.31'
	cur = con.cursor()
	cur.execute("SELECT p.value FROM v_props p JOIN components c ON (c.component_id = p.component_id) JOIN packages pkg ON (pkg.pkg_id = c.pkg_id) WHERE c.host_id = %s AND pkg.name='jdk' and p.name='java_home'", host_id)

	row = cur.fetchone()
	if row:
		java_home = row[0]
	cur.execute("SELECT p.value FROM v_props p JOIN components c ON (c.component_id = p.component_id) JOIN packages pkg ON (pkg.pkg_id = c.pkg_id) WHERE c.host_id = %s AND pkg.name = 'jakarta-tomcat' AND p.name = 'tomcat.dir'", host_id)

	row = cur.fetchone()
	if row:
		tomcat_home = row[0]

	return { 'java_home' : java_home, 'tomcat_home' : tomcat_home }

def stopUI(host_id, con):
	platform, rootpath = getHostInfo(con, host_id)

	if platform.os == 'Win32':
		hcl = win_stopUI_HCL
	else:
		version = PEMVersion.getPEMVersion()
		if version < "2.6.5":
			hcl = linux_stopUI_HCL_old % getUIStartupInfo(host_id, con)
		else:
			hcl = linux_stopUI_HCL

	return nothrowable(performHCLRequest)(host_id, hcl)

stopUI = default_ro_sysdb_con(stopUI)

def startUI(host_id, con):
	platform, rootpath = getHostInfo(con, host_id)
	if platform.os == 'Win32':
		hcl = win_startUI_HCL
	else:
		version = PEMVersion.getPEMVersion()
		if version < "2.6.5":
			hcl = linux_startUI_HCL_old % getUIStartupInfo(host_id, con)
		else:
			hcl = linux_startUI_HCL

	retriable(performHCLRequest)(host_id, hcl)


startUI = default_ro_sysdb_con(startUI)

startup_directory = os.getcwd()

__all__ = [ 'startMN', 'stopMN', 'wait_for_sc', 'execCommand', 'getSCProperty', 'wait_for_scs', 'setSCProperty', 'getHostInfo', 'getMNInfo', 'getPlatformLine', 'installation_id', 'startup_directory', 'performHCLRequest', 'startUI', 'stopUI', 'renameSC' ]
