#!/usr/bin/python
__rcs_id__ = """$Id: uSVN.py 662027 2011-06-01 10:34:29Z malpinsky $"""
__pychecker__ = "blacklist=modules/sets.py unusednames=__rcs_id__,dummy"
import os
import sys
import uUtil
import uConfig

class SVN:
	def __init__(self, config):
		self.user = config.svn.user
		self.password = config.svn.password
		self.configdir = config.svn.configdir
		self.baseurl = config.svn.baseurl
		if not self.baseurl.endswith('/'):
			self.baseurl += '/'


	def _cmd(self):
		rv = ["svn"]

		if self.user:
			rv += ["--username", self.user]
		if self.password:
			rv += ["--password", self.password]
		if self.configdir:
			rv += ["--config-dir", self.configdir]

		return rv

	def perform(self, *args):
		return uUtil.execCommand(self._cmd() + list(args))

	def read_cmd(self, *args):
		return uUtil.readCmd(self._cmd() + list(args))
		

	def list(self, directory):
		return [ x.strip() for x in self.read_cmd("ls", self.baseurl + directory).split('\n') if x.strip() ]

	def checkout(self, directory, to, *options):
		self.perform("checkout", self.baseurl + directory, to, *options)

	def update(self, directory, *args):
		cwd = os.getcwd()
		try:
			os.chdir(directory)
			self.perform("update", *args)
		finally:
			os.chdir(cwd)

	def add(self, what):
		self.perform("add", what)

	def commit(self, what, message):
		self.perform("commit", "-m", message, what)

	def mkdir(self, directory, message):
		self.perform("mkdir", "-m", message, self.baseurl + directory)

	def propset(self, filename, propname, propvalue):
		self.perform("propset", propname, propvalue, filename)

if __name__ == "__main__":
	config = uConfig.Config(uConfig.getCommonConfigArgs() + [
		('svn:user', str, None, None),
		('svn:password', str, None, None),
		('svn:configdir', str, None, None),
		('svn:baseurl', str, None, "https://svn.parallels.com/parallels/")

		], sys.argv, None, '/root/.pem/updater.cfg', (None, None),
		"""Usage: %s [[--database:host|-h] sysdb_hostname] [[--log:file|-l] logfile] [ [directory [directory...]] | --only-actions udlfile action [action..]]
	""")
	svn = SVN(config)

	for a in config.args:
		print svn.list(a)
