#!/usr/bin/python
__rcs_id__ = """$Id: uSVNBuild.py 663197 2011-06-05 12:21:44Z vvinogradov $"""
__pychecker__ = "blacklist=modules/sets.py unusednames=__rcs_id__,dummy"

import re, tempfile, shutil, uUtil, os
from uSVN import SVN

build_pattern = re.compile(r'(.+)([.-])(\d+)')

builds_dir = 'pem/pem/builds/'

def parseBuildname(buildname):
	bmatch = build_pattern.match(buildname)
	if not bmatch:
		raise Exception("%s: bad build name. Build name should end with . or -, followed by digits" % buildname)

	return bmatch.group(1), bmatch.group(3)

def loadBuild(config, buildname):
	svn = SVN(config)
	builddir, number = parseBuildname(buildname)
	tempdir = tempfile.mkdtemp()
	try:
		svndir = builds_dir + builddir + '/'
		svn.checkout(svndir, tempdir, '--depth', 'empty')
		fn = buildname + '.dat'
		svn.update(tempdir, fn)
		result = uUtil.loadObj(tempdir, fn, lambda: None)
		if not result:
			raise Exception("Failed to load %s build info from SVN. Is build name correct?" % buildname)

		return result
	finally:
		shutil.rmtree(tempdir)

def saveBuild(config, build, buildname):
	svn = SVN(config)
	builddir, number = parseBuildname(buildname)
	tempdir = tempfile.mkdtemp()
	try:
		lst = svn.list(builds_dir)
		svndir = builds_dir + builddir + '/'
		if builddir + '/' not in lst:
			svn.mkdir(svndir, "Create directory to store %s builds" % builddir)

		svn.checkout(svndir, tempdir, '--depth', 'empty')

		fn = buildname + '.dat'
		lst = svn.list(builds_dir + builddir)
		is_first_commit = False
		if fn in lst:
			svn.update(tempdir, fn)
		else:
			is_first_commit = True

		uUtil.saveObj(build, tempdir, fn)
		
		full_fn = os.path.join(tempdir, fn)
		if is_first_commit:
			svn.add(full_fn)
			svn.propset(full_fn, "svn:mime-type", "application/octet-stream")

		svn.commit(full_fn, "Save build '%s' info" % buildname)
	finally:
		shutil.rmtree(tempdir)
