import uSysDB, uPEM, openapi, PEMVersion, uTasks

def updateConfigFileInDB(con, relative_path, filename, content):
	cur = con.cursor()
	cur.execute(
	    "SELECT config_file_id FROM proxies_config_files "
	    "WHERE relative_dir = %s AND filename = %s",
	    relative_path, filename)
	row = cur.fetchone()
	if row is None:
		cur.execute(
		    "INSERT INTO proxies_config_files "
		    "(relative_dir, filename, content) VALUES (%s, %s, %s)",
		    relative_path, filename, uSysDB.toRaw(content))
		config_file_id = uSysDB.get_last_inserted_value(
		    con, "proxies_config_files")
		cur.execute(
		    "INSERT INTO proxies_config_file_statuses "
		    "(proxy_id, config_file_id) "
		    "SELECT p.proxy_id, c.config_file_id "
		    "FROM proxies p, proxies_config_files c "
		    "WHERE p.ptype = %s "
		    "AND c.config_file_id = %s", 'E', config_file_id)
	else:
		config_file_id = row[0]
		cur.execute(
		    "UPDATE proxies_config_files SET content = %s "
		    "WHERE config_file_id = %s",
		    uSysDB.toRaw(content), config_file_id)
		cur.execute(
		    "UPDATE proxies_config_file_statuses "
		    "SET updates_requested = updates_requested + 1 "
		    "WHERE config_file_id = "
		    "  (SELECT config_file_id "
		    "   FROM proxies_config_files "
		    "   WHERE config_file_id = %s "
		    "  )", config_file_id)

def scheduleConfigurationUpdate():
	uPEM.wait_for_scs(['OpenAPI', 'proxy_manager', 'TaskManager'])
	api = openapi.OpenAPI()
	api.pem.scheduleUpdates()

def scheduleBrandHtaccessDump(cur, brand_id):
	cur.execute(
	    "UPDATE brands "
	    "SET act_status ='u', act_scheduled = act_scheduled + 1 "
	    "WHERE brand_id = %s", brand_id)
	cur.execute(
	    "UPDATE brand_proxy_params "
	    "SET ht_dump_required = ht_dump_required + 1 "
	    "WHERE brand_id = %s", brand_id)

def scheduleBrandsHtaccessDump(cur):
	cur.execute(
	    "UPDATE brands "
	    "SET act_status ='u', act_scheduled = act_scheduled + 1")
	cur.execute(
	    "UPDATE brand_proxy_params "
	    "SET ht_dump_required = ht_dump_required + 1")
	version = PEMVersion.getPEMVersion()
	if version >= "2.6.9":
		task = uTasks.Task (uTasks.SCRef("brands", "0.1"), "Access points monitor", "taskReconfigureProxy")
		task.schedule()


__all__ = [
	'updateConfigFile',
	'scheduleConfigurationUpdate',
	'scheduleBrandHtaccessDump',
	'scheduleBrandsHtaccessDump' ]
