from __future__ import generators
import sys, os, re, shutil

import uLogging
import tempfile, pickle

try:
        import subprocess as sp
except ImportError:
        # for temporary backward compatibility with centos 4 and python 2.3
        import pasubprocess as sp

be_quiet = False

devnul_file = None

if sys.platform == 'win32':
	startup_info = sp.STARTUPINFO()
	startup_info.dwFlags = sp.STARTF_USESHOWWINDOW
	startup_info.wShowWindow = sp.SW_HIDE
else: # shut up pychecker
	startup_info = None

class DictToStruct:
	def __init__(self, dict):
		self.dict = dict

	def __getattr__(self, name):
		return self.dict[name]

class CounterCallback:
	def __init__(self):
		self.total = 0
		self.processed = 0

	def set_total(self, total):
		self.total = total

	def new_item(self, dummy):
		self.processed += 1

	def percents(self):
		return 100 * self.processed / self.total

def devnull():
	global devnul_file
	if devnul_file and not devnul_file.closed():
		pass
	elif sys.platform == 'win32':
		devnul_file = file('NUL', 'rw')
	else:
		devnul_file = file('/dev/null', 'rw')
	return devnul_file
	

def execCommand(command, valid_codes = None):
	uLogging.notice("Executing command: '%s'", command)

	if valid_codes is None:
		valid_exit_codes = [ 0 ]
	else:
		valid_exit_codes = valid_codes

	stdout = stderr = None
	if be_quiet:
		stderr = sp.STDOUT
		if uLogging.logfile:
			stdout = uLogging.logfile
		else:
			stdout = devnull()
	use_shell = type(command) not in (tuple, list)


	if sys.platform != 'win32':
		cmd = sp.Popen(command, stdout = stdout, stderr=stderr, close_fds = True, shell = use_shell)
	else:
		cmd = sp.Popen(command, stdout = stdout, stderr=stderr, shell = use_shell, startupinfo=startup_info)
	status = cmd.wait()

	if status >= 0 and status not in valid_exit_codes:
		raise Exception("%s exited with non-zero status %s" % (command, status))
	elif status < 0:
		raise Exception("%s is terminated by signal %s" % (command, status))
	return status

def readCmdExt(command):
	uLogging.debug("executing %s", command)
	if sys.platform != 'win32':
		cmd = sp.Popen(command, close_fds = True, stdout = sp.PIPE, stderr = sp.PIPE)
	else:
		cmd = sp.Popen(command, stdout = sp.PIPE, stderr = sp.PIPE, startupinfo = startup_info)

	out, err = cmd.communicate()
	status = cmd.returncode

	return str(out), str(err), status

def readCmd(command):
	out, err, status = readCmdExt(command)
	if status > 0:
		raise Exception("%s exited with non-zero status %s" % (command, status))
	elif status < 0:
		raise Exception("%s is terminated by signal %s" % (command, status))
	return out

	

def findHome():
	if sys.platform == 'win32':
		return os.environ.get('HOMEDRIVE', 'C:') + os.environ.get('HOMEPATH', '\\')
	else:
		return os.environ.get('HOME', '/root')


class PEMHost:
	def __init__(self, host_id, name, htype, platform, rootpath, pleskd_id = None):
		self.host_id = host_id
		self.name = name
		self.type = htype
		self.platform = platform
		self.rootpath = rootpath
		self.pleskd_id = pleskd_id
	
	def __str__(self):
		return "%s(%s)" % (self.name, self.host_id)

__prop_line_pattern = re.compile("^([^=]+)=(.*)$")

def readPropertiesFile(f, fail_on_errors = True):
	rv = {}
	line_no = 1
	for line in f:
		if line.strip().startswith('#') or not line.strip():
			continue
		pmatch = __prop_line_pattern.match(line)
		if pmatch is None:
			errmsg = "%d: '%s' bad property line" % (line_no, line)
			if fail_on_errors:
				raise Exception(errmsg)
			else:
				uLogging.err("%s", errmsg)

		else:
			rv[pmatch.group(1)] = pmatch.group(2)
		line_no += 1

	return rv;

def readPropertiesResultSet(rows):
	rv = {}
	for r in rows:
		rv[r[0]] = r[1]
	
	return rv

if sys.platform == 'win32':
	import win32api, win32con, win32file
	def moveFile(src, dst):
		return win32api.MoveFileEx(src, dst, win32file.MOVEFILE_REPLACE_EXISTING)

	def ln_cp(src, dst):
		try:
			if os.path.isdir(dst):
				dst = os.path.join(dst, os.path.basename(src))
			if os.path.isfile(dst):
				win32api.SetFileAttributes(dst, win32con.FILE_ATTRIBUTE_NORMAL)
				os.unlink(dst)

			shutil.copy2(src, dst)
		except Exception, e:
			uLogging.err("%s while copying %s to %s", e.__class__, src, dst)
			raise
else:
	def moveFile(src, dst):
		os.rename(src, dst)

	def ln_cp(src, dst):
		try:
			if os.path.isdir(dst):
				dst = os.path.join(dst, os.path.basename(src))
			if os.path.isfile(dst):
				os.unlink(dst)

			os.link(src, dst)
		except OSError:
			shutil.copy2(src, dst)

def stmt_parser(code, concatOperator, nowfun):
# it is translated to python function parseSQL from "src/library/pem_client/sqltext.cpp"
	next = ''
	quote = None
	haveSomething = eos = False
	p = 0
	while p < len(code):
		# skip comment
		if quote == None and code[p:p+2] == '--':
			p += 2
			while p < len(code) and code[p] != '\n':
				p += 1
			if p >= len(code):
				break
		if not code[p].isspace():
			haveSomething = True
		if code[p] == "'" :
			if quote == "'":
				quote = None
			elif quote == None:
				quote = "'"
		elif code[p] == '"':
			if quote == '"':
				quote = None
			elif quote == None:
				quote = '"'
		elif code[p:p+2] == '$$':
			if quote == '$$':
				quote = None
			elif quote == None:
				quote = '$$'
			next += '$'
			p+=1
		elif code[p:p+2] == '/*' and quote is None:
			quote = '/*'
			p+=2
			next += '/*'
		elif code[p:p+2] == '*/' and  quote == '/*':
			quote = None
			p+=2
			next += '*/'
		elif code[p:p+2] == '||' and quote is None:
			next += concatOperator
			p += 2
		elif code[p:p+3] == 'now' and quote is None:
			next += nowfun
			p += 3
		elif code[p] == ';':
				if quote is None:
					if haveSomething:
						eos = True
						yield next.strip()
					next = ''
					haveSomething = False
					p+=1
					continue
		if eos and not code[p].isspace():
			eos = False
			next += code[p]
		else:
			next += code[p]
		p+=1
	if haveSomething:
		yield next.strip()


def os_random_byte(dummy):
	return os.urandom(1)

def read_byte(f):
	return f.read(1)

def generate_random_password(length):
	arg = None
	if os.__dict__.has_key('urandom'):
		random_byte_fun = os_random_byte
	else:
		random_byte_fun = read_byte
		arg = open('/dev/urandom')

	rv = random_byte_fun(arg)
	while not rv.isalpha():
		rv = random_byte_fun(arg)

	while len(rv) < length:
		ch = random_byte_fun(arg)
		if ch.isalpha() or ch.isdigit():
			rv += ch
	if arg:
		arg.close()

	return rv

def saveObj(obj, dirname, filename):
	fd, tmpfn  = tempfile.mkstemp(dir = dirname)
	f = os.fdopen(fd, "w")
	pickle.dump(obj, f)
	f.close()
	target = os.path.join(dirname, filename)
	moveFile(tmpfn, target)
	os.chmod(target, 0644)

def loadObj(dirname, filename, default):
	ffname = os.path.join(dirname, filename)
	if not os.path.exists(ffname):
		return default()
	else:
		return pickle.load(open(ffname))


def editFileSafe(filename, fun, backup_filename, *args, **kwds):
	inf = outf = None
	try:
		fstat = os.stat(filename)
		fd, tmpfn  = tempfile.mkstemp(dir = os.path.dirname(filename))
		outf = os.fdopen(fd, "w")
		inf = open(filename)
		rv = fun(inf, outf, *args, **kwds)
		if backup_filename:
			ln_cp(filename, backup_filename)

		try:
			os.chmod(tmpfn, fstat.st_mode)
			os.chown(tmpfn, fstat.st_uid, fstat.st_gid)
		except:
			pass

		moveFile(tmpfn, filename)
		return rv
	finally:
		if outf:
			outf.close()
		if inf:
			inf.close()


def createPropertiesFile(properties, filename, mode = 0644):
	fd, tmpfn  = tempfile.mkstemp(dir = os.path.dirname(filename))
	f = os.fdopen(fd, "w")
	if type(properties) == dict:
		to_iter = [x for x in properties.iteritems() ]
		to_iter.sort()
	else:
		to_iter = properties
	for prop, value in to_iter:
		f.write('%s=%s\n' % (prop, value))
	
	f.close()
	os.chmod(tmpfn, mode)
	moveFile(tmpfn, filename)


def doAppendToFile(inf, outf, portion, markers = None):
	if markers is not None:
		begin, end = markers
	else:
		begin = end = None

	existing = []
	in_section = False
	for line in inf.readlines():
		if not in_section:
			if line == begin:
				in_section = True
			else:
				print >> outf, line,
		else:
			if line == end:
				in_section = False
			else:
				existing.append(line)
	
	portion_lines = portion.splitlines()
	same = True
	if len(portion_lines) == existing:
		for i in xrange(0, len(existing)):
			if existing[i].strip() != portion_lines[i].strip():
				same = False
				break
	else:
		same = False

	if not same:
		if begin:
			print >> outf, begin
		for line in portion_lines:
			print >> outf, line
		if end:
			print >> outf, end
	return not same

def appendToFile(filename, portion, markers = None):
	return editFileSafe(filename, doAppendToFile, None, portion, markers)


def startShell():
	shell_map = { 'win32' : ('COMSPEC', 'command.com'), None : ('SHELL', '/bin/sh') }
	shell = shell_map.get(sys.platform, shell_map.get(None))
	shell = os.environ.get(*shell)
	return os.system(shell)

def ln_cp_recursive(src, dst):
	if os.path.isdir(src):
		if not os.path.isdir(dst):
			os.mkdir(dst)
		for fn in os.listdir(src):
			ln_cp_recursive(os.path.join(src, fn), os.path.join(dst, fn))
	else:
		ln_cp(src, dst)


__all__ = [ "execCommand", "readCmd", "readCmdExt", "findHome", "PEMHost", "readPropertiesFile", "moveFile", "ln_cp", "stmt_parser", "saveObj", "loadObj", "startShell", "DictToStruct" ]

