import uSysDB, openapi

from uGenerics import *
from uLogging import notice, err

import PEMVersion

__installed = None
__installTemplatesSupported = None

PKG_TYPE_EZ=2
PKG_TYPE_STD=1

def installed():
	global __installed
	if __installed is None:
		con = uSysDB.connect()
		cur = con.cursor()
		cur.execute("SELECT 1 FROM service_classes WHERE name='VPSManager'")
		__installed = bool(cur.fetchone())

	return __installed

def isInstallTemplatesSupported():
	global __installTemplatesSupported
	if __installTemplatesSupported is None:
		__installTemplatesSupported = installed() and PEMVersion.getSCVersion('VPSManager') >= '5.0.0.42'
	return __installTemplatesSupported

class VZNotInstalled(Exception):
	def __init__(self):
		Exception.__init__(self, "VPS Manager is not installed")

def template_installed(host_id, templ):
	if not installed():
		return False

	con = uSysDB.connect()
	cur = con.cursor()
	cur.execute("SELECT 1 FROM ve_templates vt JOIN vz_templates t ON (t.template_id = vt.template_id) WHERE t.name = %s AND vt.host_id = %s", templ, host_id)
	return bool(cur.fetchone())

def import_template(tmpl_url):
	if not installed():
		raise VZNotInstalled()

	api = openapi.OpenAPI()
	try:
		template_id = api.pem.virtuozzo.importTemplate(url=tmpl_url, platform="Linux")['template_id']
	except openapi.OpenAPIError, e:
		if e.module_id != "VPSTemplateManagement" or e.extype_id != 8:
			raise
		template_id = int(e.properties["template_id"])
	return template_id


def list_template_urls():
	if not installed():
		return []
	con = uSysDB.connect()
	cur = con.cursor()
	cur.execute("SELECT url FROM vz_templates WHERE url IS NOT NULL")

	return [row[0] for row in cur.fetchall()]
