import pyodbc, os, re, sys
import uUtil
import MSIInstallationProgress
import uLogging
from uAction import *

class MSILogger:
	def __init__(self, file):
		self.file = open(file, "a", 0)
		self.buf = ''

	def write(self, text):
		# log to file
		self.file.write(text)

		# append to line buffer and log full lines
		self.buf += text
		lines = self.buf.splitlines(True)
		self.buf = ''
		for l in lines:
			if l.endswith('\n'):
				MSIInstallationProgress.msi_log(l.rstrip('\n'))
			else:
				self.buf += l

def setupMSILogging(file):
	logger = MSILogger(file)
	sys.stdout = logger
	sys.stderr = logger

class InstallationProgress:
	def __init__(self):
		self.phase = None
		self.ticks = 0

	def new_phase(self, phase):
		self.phase = phase
		self.ticks = 0
		MSIInstallationProgress.reset_progress()
		MSIInstallationProgress.set_message(phase)
	
	def end_phase(self):
		self.phase = None
		self.ticks = 0
		
	def set_progress(self, pval, what = ""):
		if what:
			if self.phase:
				msg = self.phase + ': '
			else:
				msg = ''
			msg += what
			MSIInstallationProgress.set_message(msg)
		ticks = pval - self.ticks
		MSIInstallationProgress.increment_progress(ticks)
		self.ticks += ticks

def getProgress():
	return InstallationProgress()


def createDatabaseAndUser(servername, admin_user, admin_password, dbname, usename, passwd):
	connect_string = "DRIVER={SQL Server Native Client 10.0}; SERVER=%s; DATABASE=master; UID=%s; PWD=%s" % (servername, admin_user, admin_password)
	con = pyodbc.connect(connect_string, autocommit=True)
	cur = con.cursor()
	try:
		cur.execute("DROP database %s" % dbname)
	except Exception, e:
		print e
	except:
		pass

	try:
		cur.execute("EXEC sp_droplogin %s" % usename)
	except Exception, e:
		print e
	except:
		print "Failed to drop", usename

	cur.execute("EXEC sp_addlogin @loginame='%s', @passwd='%s'" % (usename, passwd))
	cur.execute("CREATE DATABASE %s" % dbname)
	cur.execute("ALTER DATABASE [%s] SET READ_COMMITTED_SNAPSHOT ON" % dbname)

	cur.execute("ALTER DATABASE %s SET ARITHABORT ON" % dbname)
	cur.execute(" EXEC [%s]..sp_changedbowner '%s'" % (dbname, usename))
        cur.execute("EXEC sp_defaultdb @loginame = '%s', @defdb = '%s'" % (usename, dbname))
        cur.execute("CREATE TABLE [%s]..dual (dummy char(1))" % dbname)
        cur.execute("INSERT INTO [%s]..dual (dummy) VALUES ('X')" % dbname)
	con.commit()
	con.close()


def createDBAndUser(props):
	print "Creating database und user"
	createDatabaseAndUser(props['db_hostname'], props["admin_db_user"], props["admin_db_passwd"], props["db"], props["dsn_login"], props["dsn_passwd"])


def initSysLog(props):
	print "Creating syslog configs"
	rootpath = props['rootpath']
	syslog_etc = os.path.join(rootpath, 'syslog', 'etc')
	syslog_conf = os.path.join(syslog_etc, 'syslog.conf')
	syslog_dir = os.path.join(rootpath, 'var', 'log')
	try:
		os.makedirs(syslog_etc)
	except:
		pass # probably already exists.
	f = open(syslog_conf, 'w')
	f.write("""<?xml version="1.0"?>
<conf>
	<source name="src_udp" type="udp"/>
	<destination name="poa_log"   file="poa.log"       rotate="daily" backlogs="28" dateext="yes" autocompress="yes"/>
	<destination name="poa_debug" file="poa.debug.log" rotate="daily" backlogs="14" dateext="yes" autocompress="yes"/>
	<filter name="poa_log">
		<facility name="local0" />
		<priority name="emerg" />
		<priority name="alert" />
		<priority name="crit" />
		<priority name="error" />
		<priority name="warning" />
		<priority name="notice" />
		<priority name="info" />
	</filter>
	<filter name="poa_debug">
		<facility name="local0"/>
		<priority name="emerg"/>
		<priority name="alert"/>
		<priority name="crit"/>
		<priority name="error"/>
		<priority name="warning"/>
		<priority name="notice"/>
		<priority name="info"/>
		<priority name="debug"/>
	</filter>
	<logpath source="src_udp" filter="poa_log" destination="poa_log" />
	<logpath source="src_udp" filter="poa_debug" destination="poa_debug"/>
	<options logdir="%(syslog_dir)s" />
</conf>
""" % locals())
	f.close()

tarball_pattern = re.compile(r'.+\.(tar\.gz|tgz)$');
def copy_tarballs(to, dirname, fnames):
	for fn in fnames:
		if tarball_pattern.match(fn):
			uUtil.ln_cp(os.path.join(dirname, fn), to)


def createPPMSite(props):
	progress.do("creating ppm site")
	system_root = os.environ['SystemRoot']
	appcmd = os.path.join(system_root, 'System32', 'inetsrv', 'appcmd.exe')
	source_dir = props['source_dir']
	site_name = "Initial PEM tarballs storage"
	tarball_dir = os.path.join(props['rootpath'].replace('/', '\\'), 'install', 'tarballs')
	user = props['ipemts_user']
	password = props['ipemts_passwd']
	port = props['ipemts_port']
	ip = props['communication_ip']
	if not os.path.exists(appcmd):
		uLogging.debug("IIS 6")
		out, err, status = uUtil.readCmdExt([os.path.join(source_dir, 'os', 'Win32', 'IISAdministrationTools', '_install.bat')])
		uLogging.debug("%s %s %s", out, err, status)
		iis_web = os.path.join(system_root, 'System32', 'iisweb.vbs')
		out, err, status = uUtil.readCmdExt(['cscript', iis_web, '/delete', site_name])
		uLogging.debug("%s %s %s", out, err, status)
		out, err, status = uUtil.readCmdExt(['cscript', iis_web, '/create', tarball_dir, site_name, '/b', str(port), '/i', ip])
		uLogging.debug("%s %s %s", out, err, status)
		out, err, status = uUtil.readCmdExt(['cscript', os.path.join(system_root, 'System32', 'iisvdir.vbs'), '/create', site_name, 'tarballs',  tarball_dir])
		uLogging.debug("%s %s %s", out, err, status)
	else:
		uLogging.debug("IIS 7")
		out, err, status = uUtil.readCmdExt('%s delete site /site.name:"%s"' % (appcmd, site_name))
		uLogging.debug("%s %s %s", out, err, status)
		out, err, status = uUtil.readCmdExt('%s add site "/name:%s" /bindings:"http://%s:%s" "/physicalPath:%s"' % (appcmd, site_name, ip, port, tarball_dir))
		uLogging.debug("%s %s %s", out, err, status)
		out, err, status = uUtil.readCmdExt('%s add vdir "/app.name:%s/" "/physicalPath:%s" /path:/tarballs' % (appcmd, site_name, tarball_dir))
		uLogging.debug("%s %s %s", out, err, status)

	out, err, status = uUtil.readCmdExt(['net', 'user', user, password, '/ADD', '/EXPIRES:NEVER'])
	uLogging.debug("%s %s %s", out, err, status)
	try:
		os.makedirs(tarball_dir)
	except:
		pass
	out, err, status = uUtil.readCmdExt(["cscript", os.path.join(props['rootpath'], 'install', 'tarball_storage_config.vbs'), site_name, user, password])
	uLogging.debug("%s %s %s", out, err, status)
	progress.done()

	progress.do("copying tarballs")
	os.path.walk(source_dir, copy_tarballs, tarball_dir)
	progress.done()

	out, err, status = uUtil.readCmdExt('wmic UserAccount Where Name="%(ipemts_user)s" Set PasswordExpires=False' % props)
	uLogging.debug("%s %s %s", out, err, status)